
#include "filesetup.h"

#include "var.h"
#include "matrix.h"
#include "m_array.h"
#include "rand.h"

#include <string.h>

/******************************************************************************/
/********************************* Utilities **********************************/
/******************************************************************************/
/*
   Finds /ch or -ch in the command line.  If found returns the index
   and zero otherwise.
*/
int ParseCommandLine(int n_arg, char **args, char ch)
{
 int i;
 for (i=1; i < n_arg; i++)
  if (((args[i][0] == '/') || (args[i][0] == '-')) && (args[i][1] == ch))
   return i;
 return 0;
}

/*
   Finds /ch or -ch in the command line.  If found returns a pointer 
   to the string trailing /ch or -ch.  If /ch or -ch is not found or 
   there is no trailing string or the trailing string is another argument, 
   then default_return is returned.  No memory is allocated and 
   the calling routine should not free the returned pointer.
*/
char* ParseCommandLine_String(int n_arg, char **args, char ch, char *default_return)
{
 int i=ParseCommandLine(n_arg,args,ch);
 if (i > 0)
  if (strlen(args[i]) > 2)
    return args[i]+2;
   else
    if ((i+1 < n_arg) && (args[i+1][0] != '/') && (args[i+1][0] != '-'))
     return args[i+1];
 return default_return;
}

/*
   Finds /ch or -ch in the command line.  If found returns the integer
   value of the string trailing /ch or -ch.  If /ch or -h is not found 
   or there is no trailing string or the trailing string is another argument, 
   then the default_return value is returned.
*/
int ParseCommandLine_Integer(int n_arg, char **args, char ch, int default_return)
{
 char *str=ParseCommandLine_String(n_arg,args,ch,(char*)NULL);
 return str ? atoi(str) : default_return;
}

/*
   Returns 1 if the null terminated string id is found
   at the beginning of a line in the file and 0 otherwise.  
   The file pointer is set to the line immediately after 
   the line containing id.  Compares a maximum of 1023 
   characters of id.
*/
int SetFilePosition(FILE *f, char *id)
{
 char buffer[1024], ch;
 int n=strlen(id);
 if (n > 1023) n=1023;
 rewind(f);
 while (fgets(buffer,1024,f))
  {
   if (buffer[strlen(buffer)-1] != '\n')
    do 
     ch=fgetc(f);
    while ((ch != '\n') && (ch != EOF));
   if (!memcmp(buffer,id,n)) return 1;
  }
 return 0;
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
void FreeFileSetup(file_setup_type *setup)
{
 FreeMatrixArray(setup->A);
 FreeVector(setup->C);
 FreeMatrix(setup->InitialValues);

 FreeMatrix(setup->A_true);
 FreeMatrix(setup->F_true);
 FreeMatrix(setup->A_hat);
 FreeMatrix(setup->F_hat);

 FreeMatrixArray(setup->Q);
 FreeMatrix(setup->EC);
 FreeMatrix(setup->EP);

 free(setup);
}

file_setup_type* SetupFromFile(FILE *f_in)
{
 file_setup_type *setup;
 int i, j, k, constant;

 if (!(setup=(file_setup_type*)malloc(sizeof(file_setup_type))))
  return (file_setup_type*)NULL;

 setup->A=(TMatrix*)NULL;
 setup->C=(TVector)NULL;
 setup->A_true=(TMatrix)NULL;
 setup->F_true=(TMatrix)NULL;
 setup->InitialValues=(TMatrix)NULL;
 setup->Q=(TMatrix*)NULL;
 setup->EC=(TMatrix)NULL;
 setup->EP=(TMatrix)NULL;
 setup->A_hat=(TMatrix)NULL;
 setup->F_hat=(TMatrix)NULL;
 setup->parameterization=0;

 /* Number of variables, lags, observations and constant term */
 if (!SetFilePosition(f_in,"/* Number Variables */")
            || (fscanf(f_in," %d ",&(setup->n_var)) != 1)) goto ERROR;

 if (!SetFilePosition(f_in,"/* Number Lags */")
            || (fscanf(f_in," %d ",&(setup->n_lag)) != 1)) goto ERROR;

 if (!SetFilePosition(f_in,"/* Constant Term (0 - No   1 - Yes) */")
            || (fscanf(f_in," %d ",&(constant)) != 1)) goto ERROR;

 if (!SetFilePosition(f_in,"/* Number Observations */")
            || (fscanf(f_in," %d ",&(setup->n_obs)) != 1)) goto ERROR;

 if ((setup->n_var <= 0) || (setup->n_lag < 0) || (setup->n_obs) <= 0)
  goto ERROR;

 setup->n_exg=constant ? 1 : 0;
 setup->n_pre=setup->n_var*setup->n_lag+setup->n_exg;

 /* True Parameters */
 if (!SetFilePosition(f_in,"/* A[0] A[1] ... A[Number Lags] Constant */"))
  goto ERROR;
 if (!(setup->A=CreateMatrixArray(setup->n_lag+1))) goto ERROR;
 for (i=0; i <= setup->n_lag; i++)
  if (!ReadMatrix(f_in,setup->A[i]=CreateMatrix(setup->n_var,setup->n_var)))
   goto ERROR;
 if (constant)
  if (!ReadVector(f_in,setup->C=CreateVector(setup->n_var))) goto ERROR;

 if (!SetFilePosition(f_in,"/* A[0] A[1] ... A[Number Lags] Constant */")
       || !ReadMatrix(f_in,setup->A_true=CreateMatrix(setup->n_var,setup->n_var))
            || (setup->n_pre && !ReadMatrix(f_in,setup->F_true=CreateMatrix(setup->n_pre,setup->n_var))))
  goto ERROR;

 /* Initial Values */
 if ((setup->n_lag > 0) && SetFilePosition(f_in,"/* Initial Values */"))
  if (!ReadMatrix(f_in,setup->InitialValues=CreateMatrix(setup->n_lag,setup->n_var)))
   goto ERROR;

 /* Burn-in Period */
 if (!SetFilePosition(f_in,"/* Burn-in Period */")
            || (fscanf(f_in," %d ",&(setup->burn_in)) != 1)
                   || (setup->burn_in < 0))
                              setup->burn_in=0;

 /* Seed Value */
 if (!SetFilePosition(f_in,"/* Seed */")
            || (fscanf(f_in," %d ",&(setup->seed)) != 1)) setup->seed=0;

 /* Exclusion Restrictions */
 if (SetFilePosition(f_in,"/* Exclusion Restrictions */"))
  if (!ReadMatrix(f_in,setup->EC=CreateMatrix(setup->n_var,setup->n_var))
         || ((setup->n_pre > 0)
               && !ReadMatrix(f_in,setup->EP=CreateMatrix(setup->n_pre,setup->n_var))))
                     goto ERROR;

 /* Linear Restrictions */
 if (SetFilePosition(f_in,"/* Linear Restrictions */"))
  if (!(setup->Q=CreateMatrixArray(setup->n_var)))
    goto ERROR;
   else
    for (i=0; i < setup->n_var; i++)
     if ((fscanf(f_in," %d ",&k) != 1)
       || !ReadMatrix(f_in,setup->Q[i]=CreateMatrix(k,setup->n_var+setup->n_pre))) goto ERROR;

 /* Parameterization */
 if (!SetFilePosition(f_in,"/* Parameterization Type */")
          || (fscanf(f_in," %d ",&(setup->parameterization)) != 1))
                   setup->parameterization=0;

 /* Outer Sample Size */
 if (!SetFilePosition(f_in,"/* Outer Sample Size */")
          || (fscanf(f_in," %d ",&(setup->outer_loop)) != 1)
                 || (setup->outer_loop < 1))
                        setup->outer_loop=1;

 /* Inner Sample Size */
 if (!SetFilePosition(f_in,"/* Inner Sample Size */")
          || (fscanf(f_in," %d ",&(setup->inner_loop)) != 1)
                 || (setup->inner_loop < 1))
                        setup->inner_loop=1;

 /* Parameter Draws */
 if (!SetFilePosition(f_in,"/* Parameter Draws */")
          || (fscanf(f_in," %d ",&(setup->parameter_draws)) != 1)
                 || (setup->parameter_draws < 1))
                        setup->parameter_draws=1;

 return setup;

ERROR:
 FreeFileSetup(setup);
 return NULL;
}

void PrintFileSetup(FILE *f, file_setup_type *setup)
{
 int i;

 /* Number of variables */
 fprintf(f,"/* Number Variables */\n%d\n\n",setup->n_var);

 /* Number of lags */
 fprintf(f,"/* Number Lags */\n%d\n\n",setup->n_lag);

 /* Constant term */
 fprintf(f,"/* Constant Term (0 - No   1 - Yes) */\n%d\n\n",setup->n_exg);

 /* Restrictions */
 fprintf(f,"/* Exclusion Restrictions */\n");
 PrintMatrix(f,setup->EC,"%.0lf\t"); fprintf(f,"\n");
 PrintMatrix(f,setup->EP,"%.0lf\t"); fprintf(f,"\n");

 /* True parameters */
 fprintf(f,"/* A[0] A[1] ... A[Number Lags] Constant */\n");
 for (i=0; i <= setup->n_lag; i++)
  {
   PrintMatrix(f,setup->A[i],"%.2lf\t");
   fprintf(f,"\n");
  }
 if (setup->n_exg)
  {
   PrintVector(f,setup->C,"%.2lf\t");
   fprintf(f,"\n");
  }

 /* Initial values */
 if (setup->InitialValues)
  {
   fprintf(f,"/* Initial Values */\n");
   PrintMatrix(f,setup->InitialValues,"%.2lf\t");
   fprintf(f,"\n");
  }
 fprintf(f,"/* Burn-in Peroid */\n%d\n\n",setup->burn_in);
 fprintf(f,"/* Number Observations */\n%d\n\n",setup->n_obs);
 fprintf(f,"/* Seed */\n%d\n\n",setup->seed);

 /* Parameterizations */
 fprintf(f,"/****************************************************************************************/\n" 
           " Parameterization - None                    0\n" 
           "                  - WZ                      1\n" 
           "                  - Beta                    2\n" 
           "                  - Eta                     3\n" 
           "                  - Pi                      4\n" 
           "                  - Wald                    5\n" 
           "/****************************************************************************************/\n"
           "/* Parameterization Type */\n%d\n\n",setup->parameterization);

 fprintf(f,"/* Outer Sample Size */\n%d\n\n",setup->outer_loop);
 fprintf(f,"/* Inner Sample Size */\n%d\n\n",setup->inner_loop);
 fprintf(f,"/* Parameter Draws */\n%d\n\n",setup->parameter_draws);

 fprintf(f,"\n");
}

/*
   Generates data from true parameters in setup.  Does not set
   seed value.
*/
TMatrix GenerateData(file_setup_type *setup)
{
 TMatrix rtrn, Data, A_inverse=(TMatrix)NULL;
 TVector y=(TVector)NULL, v=(TVector)NULL;
 int i, j, k;

 if ((rtrn=CreateMatrix(setup->n_obs+setup->n_lag,setup->n_var))
        && (Data=CreateMatrix(setup->n_obs+setup->burn_in+setup->n_lag,setup->n_var))
        && (A_inverse=Inverse_LU((TMatrix)NULL,setup->A[0]))
        && (y=CreateVector(setup->n_var))
        && (v=CreateVector(setup->n_var)))
  {
   if (setup->n_lag > 0)
    if (setup->InitialValues)
      memcpy(Data[0],setup->InitialValues[0],setup->n_lag*setup->n_var*sizeof(PRECISION));
     else
      for (k=setup->n_lag*setup->n_var-1; k >= 0; k--) Data[0][k]=0.0;

   for (i=setup->n_lag; i < setup->n_obs+setup->n_lag+setup->burn_in; i++)
    {
     NormalVector(y);
     if (setup->C) AddVV(y,y,setup->C);
     for (j=1; j <= setup->n_lag; j++)
      {
       for (k=0; k < setup->n_var; k++) v[k]=Data[i-j][k];
       ProductVM(v,v,setup->A[j]);
       AddVV(y,y,v);
      }
     ProductVM(y,y,A_inverse);
     for (k=0; k < setup->n_var; k++) Data[i][k]=y[k];
    }
  }
 memcpy(rtrn[0],Data[setup->burn_in],(setup->n_obs + setup->n_lag)*setup->n_var*sizeof(PRECISION));
 FreeMatrix(A_inverse);
 FreeVector(y);
 FreeVector(v);
 FreeMatrix(Data);
 return rtrn;
}

/*
   Creates svar from setup (sizes, restrictions) and Data.  
   The normalization type is set to none and the ML estimates 
   are not computed.
*/
T_SVAR* SetupFromStructure(file_setup_type *setup, TMatrix Data)
{
 int i;
 T_SVAR *svar;
 TMatrix ExgData=(TMatrix)NULL;

 if (!(svar=Construct_SVAR(setup->n_var,setup->n_lag,setup->n_exg)))
  return (T_SVAR*)NULL;

 SetExclusionRestrictions(setup->EC,setup->EP,svar);
 //SetLinearRestrictions(setup->Q,svar))

 if (setup->n_exg) ExgData=InitializeMatrix(CreateMatrix(setup->n_obs,1),1.0);
 if (DataUpdate(Data,ExgData,setup->n_lag,0,setup->n_obs,-1,svar))
   {
     printf("Error in DataUpdate().\n");
     exit(0);
   }
 FreeMatrix(ExgData);

 if (CheckSimulations(svar))
  {
   Destruct_SVAR(svar);
   return (T_SVAR*)NULL;
  }

 return svar;
}

TMatrix* MakeLinearRestrictions(TMatrix EC, TMatrix EP, int n, int k)
{
 int i, j, m;
 TMatrix *Q=CreateMatrixArray(n); 
 for(j=n-1; j >= 0; j--)
  {
   m=0;
   if (EC)
    for (i=0; i < n; i++)
     if (fabs(EC[i][j]) < SQRT_MACHINE_EPSILON) m++;
   if (EP)
    for (i=0; i < k; i++)
     if (fabs(EP[i][j]) < SQRT_MACHINE_EPSILON) m++;
   if (m > 0)
    {
     Q[j]=InitializeMatrix(CreateMatrix(m,n+k),0.0);
     m=0;
     if (EC)
      for (i=0; i < n; i++)
       if (fabs(EC[i][j]) < SQRT_MACHINE_EPSILON) 
        Q[j][m++][i]=1.0;
     if (EP)
      for (i=0; i < k; i++)
       if (fabs(EP[i][j]) < SQRT_MACHINE_EPSILON) 
        Q[j][m++][n+i]=1.0;
    }
  }
 return Q; 
}


