
#include <math.h>
#include <string.h>
#include "histo.h"

static void Resize(PRECISION x, int *h, PRECISION *min, PRECISION *max, int intervals);
static void AddObservationVariable(PRECISION x, int *h, PRECISION *min, PRECISION *max, int intervals);
static void AddObservationFixed(PRECISION x, int *low, int *h, int *high, PRECISION min, PRECISION max, int intervals);

/*
     h    : ((bins+2) x 2) matrix or null pointer
	 data : array of length obs
	 obs  : number of data observations
	 bins : number of bins to use
	 
     Creates histogram h if passed null pointer.  Returns h upon success 
	 and null pointer on failure.  Interval endpoints are chosen so that 
     a equal number of observations are in each interval.  The first column
     is the mid-point of the bin and the second column is the relative 
     frequency divided by the width of the bin.  The element h[0][0] 
     contains the minimum data point and h[bins+1][0] contains the maximum
     data point.  Both h[0][1] and h[bins+1][1] are zero.
*/
static void sort(PRECISION *x, PRECISION *y, int n)
{
 int i, hi, lo, mid;
 //printf("Start\n");
 x[0]=y[0];
 for (i=1; i < n; i++)
  if (y[i] >= x[i-1])
    x[i]=y[i];
   else
    {
     if (y[i] < x[0])
       hi=0;
      else
       {
        lo=0;
        hi=i-1;
        while (hi-lo > 1)
         if (y[i] < x[mid=(hi+lo)/2])
           hi=mid;
          else
           lo=mid;

        /**printf("%lf %d\n",y[i],hi);
        for (j=0; j < i; j++) printf("%lf\n",x[j]);
        getchar();/**/ 
       }
     memmove(x+hi+1,x+hi,(i-hi)*sizeof(PRECISION));
     x[hi]=y[i];

        /**if ((hi > 0) && (x[hi-1] > x[hi]))
         {
          printf("%d %le\n%d %le\n",hi-1,x[hi-1],hi,x[hi]);
          getchar();
         }
        if ((hi+1 < n) && (x[hi] > x[hi+1]))
         {
          printf("%d %le\n%d %le\n",hi+1,x[hi+1],hi,x[hi]);
          getchar();
         }/**/

        /**for (j=0; j <= i; j++) printf("%lf\n",x[j]);
        getchar();/**/
    }
 /**printf("End\n");
 for (i=1; i < n; i++)
  if (x[i] < x[i-1])
   {
    printf("%d %le\n%d %le\n",i-1,x[i-1],i,x[i]);
    getchar();
   }/**/
}
TMatrix data_histogram(TMatrix h, PRECISION *data, int obs, int bins)
{
 PRECISION *x, lo, hi, diff;
 int i, j, k, m;
 if (!(x=(PRECISION*)malloc(obs*sizeof(PRECISION)))) return (TMatrix)NULL;
 if (!h)
   {
    if (!(h=CreateMatrix(bins+2,2))) 
	 {
	  free(x);
	  return (TMatrix)NULL;
	 }
   }
  else
   {
    if ((M_ROW(h) != bins+2) || (M_COL(h) != 2))
	 {
	  free(x);
	  return (TMatrix)NULL;
	 }
   }
 sort(x,data,obs);
 lo=h[0][0]=x[0];
 k=m=j=0;
 for (i=1; i <= bins; i++)
  if (m >= obs)
    {
     h[i][0]=x[obs-1];
     h[i][1]=0.0;
    }
   else
    {
     while (m > ((++j)*(obs-1))/bins);
     m=j*(obs-1)/bins;
     while ((m < obs-1) && (x[m+1] == x[m])) m++;
     hi=(m < obs-1) ? (x[m]+x[m+1])/2.0 : x[m];
     h[i][0]=(hi+lo)/2.0;
     if ((diff=hi-lo) <= 0) diff=MACHINE_EPSILON;
     h[i][1]=((PRECISION)(m-k+1))/(diff*(PRECISION)obs); 
     lo=hi;
     k=++m;
    }
 h[bins+1][0]=x[obs-1];
 h[0][1]=h[bins+1][1]=0.0;
 free(x);
 return h; 
}

/*
     h    : ((bins+2) x 2) matrix or null pointer
	 data : array of length obs
	 obs  : number of data observations
	 bins : number of bins to use
	 
     Creates histogram h if passed null pointer.  Returns h upon success 
	 and null pointer on failure.  Intervals are of equal length.  The 
     first column is the mid-point of the bin and the second column is the
     relative frequency divided by the width of the bin.  The element 
     h[0][0] contains the minimum data point and h[bins+1][0] contains the 
     maximum data point.  Both h[0][1] and h[bins+1][1] are zero.
*/
TMatrix data_histogram_uniform_width(TMatrix h, PRECISION *data, int obs, int bins)
{
 PRECISION max, min, inc;
 int i, j;
 if (!h)
   { if (!(h=CreateMatrix(bins+2,2))) return (TMatrix)NULL; }
  else
   { if ((M_ROW(h) != bins+2) || (M_COL(h) != 2)) return (TMatrix)NULL; } 
 max=min=data[obs-1];
 for (i=obs-2; i >= 0; i--)
  if (data[i] > max)
    max=data[i];
   else
    if (data[i] < min) min=data[i];
 h[0][0]=min;
 h[bins+1][0]=max;
 h[0][1]=h[bins+1][1]=0;
 inc=(max-min)/bins;
 for (i=bins; i > 0; i--)
  {
   h[i][0]=min+(i-0.5)*inc;
   h[i][1]=0;
  }
 if (inc > 0)
   {
    for (i=obs-1; i >= 0; i--)
     {
      j=(int)((data[i]-min)/inc);
      h[(j < bins) ? j+1 : bins][1]+=1;
     }
    for (i=bins; i > 0; i--) h[i][1]/=(inc*obs);
   }
  else
   h[1][1]=1.0;
 return h;
}


/*
     m : number of bins
     X : n x k matrix of data
     W : optional n x k matrix of weights

     Returns histogram.  For each column of data, m bins of equal size are
     formed using the actual minimum and maximum values for that column.
     The relative frequency, using the weights if available, is computed.
     Two columns are returned.  The first column is the mid-point of the
     bin and the second column is the relative frequency divided by the
     width of the bin.
*/
TMatrix default_histogram(TMatrix X, TMatrix W, int m)
{
 int i, j, n=M_ROW(X), k=M_COL(X);
 TVector min=CreateVector(k), max=CreateVector(k);
 TMatrix bins;
 PRECISION inc;

 for (j=k-1; j >= 0; j--)
  {
   min[j]=max[j]=X[0][j];
   for (i=n-1; i > 0; i--)
    if (min[j] > X[i][j])
      min[j]=X[i][j];
     else
      if (max[j] < X[i][j])
       max[j]=X[i][j];

   if (max[j] == min[j])
    {
     inc=0.5*SQRT_MACHINE_EPSILON*m;
     if (fabs(min[j]) > 1) inc*=fabs(min[j]);
     min[j]-=inc;
     max[j]+=inc;
    }
  }

 bins=histogram(X,W,m,min,max);

 FreeVector(min);
 FreeVector(max);

 return bins;
}

/*
     X   : n x k matrix of data
     W   : optional n x k matrix of weights
     m   : number of bins
     min : k-vector of minimum values
     max : k-vector of maximum values

     Returns histogram.  For each column of data, m bins of equal size are
     formed using the passed minimum and maximum values for that column.
     The relative frequency, using the weights if available, is computed.
     Two columns are returned.  The first column is the mid-point of the bin
     and the second column is the relative frequency divided by the width of
     the bin.
*/
TMatrix histogram(TMatrix X, TMatrix W, int m, TVector min, TVector max)
{
 int i, j, l, n=M_ROW(X), k=M_COL(X);
 PRECISION w;
 TMatrix bins=InitializeMatrix(CreateMatrix(m,2*k),0);
 TVector t=InitializeVector(CreateVector(k),0), inc=CreateVector(k);

 for (j=k-1; j >= 0; j--)
  {
   inc[j]=(double)m/(max[j]-min[j]);
   for (i=m-1; i >= 0; i--) bins[i][2*j]=min[j]+(i+0.5)/inc[j];
  }

 for (i=n-1; i >= 0; i--)
  for (j=k-1; j >= 0; j--)
   {
    t[j]+=(w=W ? W[i][j] : 1);

    if ((min[j] <= X[i][j]) && (X[i][j] <= max[j]))
     {
      l=(int)floor((X[i][j]-min[j])*inc[j]);
      bins[(l >= m) ? m-1 : l][2*j+1]+=w;
     }
   }

 for (j=k-1; j >= 0; j--)
  {
   inc[j]/=t[j];
   for (i=m-1; i >= 0; i--) bins[i][2*j+1]*=inc[j];
  }

 FreeVector(t);
 FreeVector(inc);

 return bins;
}

/*
     X   : n x 2 matrix of data
     W   : optional n-vector of weights
     m   : number of bins

   Returns 2d-histogram.  For each column of data, the minimum and maximum
   values are computed and stored in the vectors min and max.  Care is taken
   to ensure that min is strictly less than max.  The rectangle
   [min[0],max[0]] x [min[1],max[1]] is divided into m-squared equal
   rectangular bins.  The relative frequency, using the weights if available,
   is computed for each bin.  The relative frequency divided by the area of
   the bin is returned.  The first row and column of the matrix returned
   contains the mid-points of the bins.

   The function histogram_2D(X,W,m,min,max) is called.
*/
TMatrix default_histogram_2D(TMatrix X, TVector W, int m)
{
 int i, j, n=M_ROW(X);
 TVector min=CreateVector(2), max=CreateVector(2);
 TMatrix bins;
 PRECISION inc;

 if (M_COL(X) != 2) MatrixError(SIZE_ERR);
 if (W && (V_DIM(W) != n)) MatrixError(SIZE_ERR);

 for (j=1; j >= 0; j--)
  {
   min[j]=max[j]=X[0][j];
   for (i=n-1; i > 0; i--)
    if (min[j] > X[i][j])
      min[j]=X[i][j];
     else
      if (max[j] < X[i][j])
       max[j]=X[i][j];

   if (max[j] == min[j])
    {
     inc=0.5*SQRT_MACHINE_EPSILON*m;
     if (fabs(min[j]) > 1) inc*=fabs(min[j]);
     min[j]-=inc;
     max[j]+=inc;
    }
  }

 bins=histogram_2D(X,W,m,min,max);

 FreeVector(min);
 FreeVector(max);

 return bins;
}


/*
     X   : n x 2 matrix of data
     W   : optional n-vector of weights
     m   : number of bins
     min : 2-vector of minimum values
     max : 2-vector of maximum values

   Returns 2d-histogram.  The rectangle [min[0],max[0]] x [min[1],max[1]]
   is divided into m-squared equal rectangular bins.  The relative frequency,
   useing the weights if available, is computed for each bin.  The relative
   frequency divided by the area of the bin is returned.  The first row
   and column of the matrix returned contains the mid-points of the bins.
*/
TMatrix histogram_2D(TMatrix X, TVector W, int m, TVector min, TVector max)
{
 int i, j, k, n=M_ROW(X);
 double w=0;
 TMatrix bins=InitializeMatrix(CreateMatrix(m+1,m+1),0);
 TVector inc=CreateVector(2);

 inc[0]=(float)m/(max[0]-min[0]);
 inc[1]=(float)m/(max[1]-min[1]);

 for (i=m; i > 0; i--)
  for (j=m; j > 0; j--)
   bins[i][j]=0;

 for (k=m; k > 0; k--)
  {
   bins[k][0]=min[0]+(k-0.5)/inc[0];
   bins[0][k]=min[1]+(k-0.5)/inc[1];
  }

 bins[0][0]=0;

 for (k=n-1; k >= 0; k--)
  {
   w+=W ? W[k] : 1;

   if ((X[k][0] < min[0]) || (X[k][0] > max[0])
          || (X[k][1] < min[1]) || (X[k][1] > max[1])) break;

   i=(int)floor((X[k][0]-min[0])*inc[0])+1;
   j=(int)floor((X[k][1]-min[1])*inc[1])+1;

   if (i > m) i=m;
   if (j > m) j=m;

   bins[i][j]+=W ? W[i] : 1;
  }

 w=inc[0]*inc[1]/w;

 for (i=m; i > 0; i--)
  for (j=m; j > 0; j--)
   bins[i][j]*=w;

 FreeVector(inc);

 return bins;
}

/*******************************************************************************
  The following set of routines create a matrix of histograms on the fly.
*******************************************************************************/
/*
   Assumes
     rows > 0
     cols > 0
     intervals > 0
     type = HISTOGRAM_FIXED or  HISTOGRAM_VARIABLE

   Results
     Creates and returns a matrix histogram data structure.  The size of the
     matrix is m x n and the number of intervals is intrvls.

*/
TMatrixHistogram *CreateMatrixHistogram(int rows, int cols, int intervals, int type)
{
 int i, j;
 TMatrixHistogram *h;

 if (!(h=(TMatrixHistogram *)malloc(sizeof(TMatrixHistogram)))) MatrixError(MEM_ERR);

 if (!(h->freq=(int***)malloc(rows*sizeof(int**)))) MatrixError(MEM_ERR);
 for (i=rows-1; i >= 0; i--)
  {
   if (!(h->freq[i]=(int**)malloc(cols*sizeof(int*)))) MatrixError(MEM_ERR);
   for (j=cols-1; j >= 0; j--)
    if (!(h->freq[i][j]=(int*)malloc(intervals*sizeof(int)))) MatrixError(MEM_ERR);
  }

 if (!(h->low=(int**)malloc(rows*sizeof(int*)))) MatrixError(MEM_ERR);
  for (i=rows-1; i >= 0; i--)
   if (!(h->low[i]=(int*)malloc(cols*sizeof(int)))) MatrixError(MEM_ERR);

 if (!(h->high=(int**)malloc(rows*sizeof(int*)))) MatrixError(MEM_ERR);
  for (i=rows-1; i >= 0; i--)
   if (!(h->high[i]=(int*)malloc(cols*sizeof(int)))) MatrixError(MEM_ERR);  

 h->Min=CreateMatrix(rows,cols);
 h->Max=CreateMatrix(rows,cols);

 h->rows=rows;
 h->cols=cols;
 h->intervals=intervals;
 h->sample_size=0;
 h->type=type;

 return h;
}

void SetMaxMinMatrixHistogram(TMatrix Min, TMatrix Max, TMatrixHistogram *h)
{
 EquateMatrix(h->Min,Min);
 EquateMatrix(h->Max,Max);
 h->sample_size=0;
}

void FreeMatrixHistogram(TMatrixHistogram *h)
{
 int i, j;
 for (i=h->rows-1; i >= 0; i--)
  {
   for (j=h->cols-1; j >= 0; j--) free(h->freq[i][j]);
   free(h->freq[i]);
  }
 free(h->freq);
 for (i=h->rows-1; i >= 0; i--) free(h->low[i]);
 free(h->low);
 for (i=h->rows-1; i >= 0; i--) free(h->high[i]);
 free(h->high);
 FreeMatrix(h->Min);
 FreeMatrix(h->Max);
 free(h);
}

void AddMatrixObservation(TMatrix X, TMatrixHistogram *h)
{
 int i, j, k;

 if ((h->rows != M_ROW(X)) || (h->cols != M_COL(X))) MatrixError(SIZE_ERR);

 if (h->sample_size <= 0)
   {
     for (i=h->rows-1; i >= 0; i--)
       for (j=h->cols-1; j >= 0; j--)
	 {
	   h->low[i][j]=h->high[i][j]=0;
	   for (k=h->intervals-1; k >= 0; k--) h->freq[i][j][k]=0;
	 }
     if (h->type == HISTOGRAM_VARIABLE)
       for (i=h->rows-1; i >= 0; i--)
	 for (j=h->cols-1; j >= 0; j--)
	   h->Min[i][j]=h->Max[i][j]=X[i][j];
   }

 if (h->type == HISTOGRAM_FIXED)
   for (i=h->rows-1; i >= 0; i--)
     for (j=h->cols-1; j >= 0; j--)
       AddObservationFixed(X[i][j],h->low[i]+j,h->freq[i][j],h->high[i]+j,h->Min[i][j],h->Max[i][j],h->intervals);
 else
   for (i=h->rows-1; i >= 0; i--)
     for (j=h->cols-1; j >= 0; j--)
       AddObservationVariable(X[i][j],h->freq[i][j],h->Min[i]+j,h->Max[i]+j,h->intervals);

 h->sample_size++;
}

void MatrixPrecentile(TMatrix X, PRECISION precentile, TMatrixHistogram *h)
{
 int i, j;

 if ((h->rows != M_ROW(X)) || (h->cols != M_COL(X))) MatrixError(SIZE_ERR);

 for (i=h->rows-1; i >= 0; i--)
  for (j=h->cols-1; j >= 0; j--)
   X[i][j]=Precentile(precentile,h->low[i][j],h->freq[i][j],h->Min[i][j],h->Max[i][j],h->intervals,h->sample_size);
}

/*
   Returns the probability that an observation is less than or equal to
   level.

   Assumes
     For 0 <= i < h->rows and 0 <= j < h->cols, let

       I[i][j][k]=(h->min[i][j] + k*inc[i][j], h->min[i][j] + (k+1)*inc[i][j]),

     where inc[i][j]=(h->max[i][j] - h->min[i][j])/h->samples_size.  The
     distribution is uniform on I[i][k][j] and

      P(h->min[i][j] + k*inc[i][j] < x[i][j] < h->min[i][j] + (k+1)*inc[i][j])
                          = h->freq[i][j][k]/h->sample_size.

     Furthermore,

          P(x[i][j] < h->min[i][j]) = 0 and P(x[i][j] > h->min[i][j]) = 0.

     In addition, if h->type == FIXED, then

              P(x[i][j] = h->min[i][j]) = h->low[i][j]/h->sample_size

     and

             P(x[i][j] = h->min[i][j]) = h->high[i][j]/h->sample_size.
*/
void MatrixCumulative(TMatrix P, TMatrix Level, TMatrixHistogram *h)
{
 int i, j;

 if ((h->rows != M_ROW(P)) || (h->cols != M_COL(P)) ||
        (h->rows != M_ROW(Level)) || (h->cols != M_COL(Level)))
  MatrixError(SIZE_ERR);

 for (i=h->rows-1; i >= 0; i--)
  for (j=h->cols-1; j >= 0; j--)
   P[i][j]=Cumulative(Level[i][j],h->low[i][j],h->freq[i][j],h->Min[i][j],h->Max[i][j],h->intervals,h->sample_size);
}

TMatrix PlotMatrixHistogramAuto(int i, int j, int bins, TMatrixHistogram *h)
{
  return MakeHistogramAuto(h->low[i][j],h->freq[i][j],h->high[i][j],h->Min[i][j],h->Max[i][j],h->intervals,h->sample_size,bins);
}

TMatrix PlotMatrixHistogram(int i, int j, PRECISION min, PRECISION max, int bins, TMatrixHistogram *h)
{
  return MakeHistogram(h->low[i][j],h->freq[i][j],h->Min[i][j],h->Max[i][j],h->intervals,h->sample_size,min,max,bins);
}
/*******************************************************************************
  The following set of routines create a vector of histograms on the fly.
*******************************************************************************/
TVectorHistogram *CreateVectorHistogram(int dim, int intervals, int type)
{
 int i;
 TVectorHistogram *h;

 if (!(h=(TVectorHistogram *)malloc(sizeof(TVectorHistogram))))
  MatrixError(MEM_ERR);

 if (!(h->freq=(int**)malloc(dim*sizeof(int*)))) MatrixError(MEM_ERR);
 for (i=dim-1; i >= 0; i--)
  if (!(h->freq[i]=(int*)malloc(intervals*sizeof(int)))) MatrixError(MEM_ERR);

 if (!(h->low=(int*)malloc(dim*sizeof(int)))) MatrixError(MEM_ERR);
 if (!(h->high=(int*)malloc(dim*sizeof(int)))) MatrixError(MEM_ERR);

 h->Min=CreateVector(dim);
 h->Max=CreateVector(dim);

 h->dim=dim;
 h->intervals=intervals;
 h->sample_size=0;
 h->type=type;

 return h;
}

void SetMaxMinVectorHistogram(TVector Min, TVector Max, TVectorHistogram *h)
{
 EquateVector(h->Min,Min);
 EquateVector(h->Max,Max);
 h->sample_size=0;
}

void FreeVectorHistogram(TVectorHistogram *h)
{
 int i;
 for (i=h->dim-1; i >= 0; i--) free(h->freq[i]);
 free(h->freq);
 free(h->low);
 free(h->high);
 FreeVector(h->Min);
 FreeVector(h->Max);
 free(h);
}

void AddVectorObservation(TVector x, TVectorHistogram *h)
{
 int i, k;

 if (h->dim != V_DIM(x)) MatrixError(SIZE_ERR);

 if (h->sample_size <= 0)
   {
     for (i=h->dim-1; i >= 0; i--)
       {
	 h->low[i]=h->high[i]=0;
	 for (k=h->intervals-1; k >= 0; k--) h->freq[i][k]=0;
       }
     if (h->type == HISTOGRAM_VARIABLE)
       for (i=h->dim-1; i >= 0; i--)
	 h->Min[i]=h->Max[i]=x[i];
   }

 if (h->type == HISTOGRAM_FIXED)
   for (i=h->dim-1; i >= 0; i--)
     AddObservationFixed(x[i],h->low+i,h->freq[i],h->high+i,h->Min[i],h->Max[i],h->intervals);
 else
   for (i=h->dim-1; i >= 0; i--)
     AddObservationVariable(x[i],h->freq[i],h->Min+i,h->Max+i,h->intervals);

 h->sample_size++;
}

void VectorPrecentile(TVector x, PRECISION precentile, TVectorHistogram *h)
{
 int i;

 if (h->dim != V_DIM(x)) MatrixError(SIZE_ERR);

 for (i=h->dim-1; i >= 0; i--)
  x[i]=Precentile(precentile,h->low[i],h->freq[i],h->Min[i],h->Max[i],h->intervals,h->sample_size);
}

/*
   Returns the probability that an observation is less than or equal to
   level.

   Assumes
     For 0 <= i < h->dim, let

             I[i][k]=(h->min[i] + k*inc[i], h->min[i] + (k+1)*inc[i]),

     where inc[i]=(h->max[i] - h->min[i])/h->samples_size.  The distribution
     is uniform on I[i][k] and

       P(h->min[i] + k*inc[i] < x[i] < h->min[i] + (k+1)*inc[i])
                                              = h->freq[i][k]/h->sample_size.

     Furthermore,

            P(x[i] < h->min[i]) = 0 and P(x[i] > h->min[i]) = 0.

     In addition, if h->type == FIXED, then

                P(x[i] = h->min[i]) = h->low[i]/h->sample_size

     and

                P(x[i] = h->min[i]) = h->high[i]/h->sample_size.
*/
void VectorCumulative(TVector p, TVector level, TVectorHistogram *h)
{
 int i;

 if (h->dim != V_DIM(p) || (h->dim != V_DIM(level)))
  MatrixError(SIZE_ERR);

 for (i=h->dim-1; i >= 0; i--)
  p[i]=Cumulative(level[i],h->low[i],h->freq[i],h->Min[i],h->Max[i],h->intervals,h->sample_size);
}

TMatrix PlotVectorHistogramAuto(int i, int bins, TVectorHistogram *h)
{
  return MakeHistogramAuto(h->low[i],h->freq[i],h->high[i],h->Min[i],h->Max[i],h->intervals,h->sample_size,bins);
}

TMatrix PlotVectorHistogram(int i, PRECISION min, PRECISION max, int bins, TVectorHistogram *h)
{
  return MakeHistogram(h->low[i],h->freq[i],h->Min[i],h->Max[i],h->intervals,h->sample_size,min,max,bins);
}
/*******************************************************************************
  The following set of routines create a scalar histogram on the fly.
*******************************************************************************/
/*
   Assumes

   Results
     Creates and returns a scalar histogram data structure.
*/
TScalarHistogram *CreateScalarHistogram(int intervals, int type)
{
 TScalarHistogram *h;

 if (!(h=(TScalarHistogram *)malloc(sizeof(TScalarHistogram)))) MatrixError(MEM_ERR);

 if (!(h->freq=(int*)malloc(intervals*sizeof(int)))) MatrixError(MEM_ERR);

 h->intervals=intervals;
 h->sample_size=0;
 h->type=type;

 return h;
}

void SetMaxMinScalarHistogram(PRECISION Min, PRECISION Max, TScalarHistogram *h)
{
 h->Min=Min;
 h->Max=Max;
 h->sample_size=0;
}

void FreeScalarHistogram(TScalarHistogram *h)
{
 free(h->freq);
 free(h);
}

void AddScalarObservation(PRECISION x, TScalarHistogram *h)
{
 int k;

 if (h->sample_size <= 0)
  {
   h->low=h->high=0;
   for (k=h->intervals-1; k >= 0; k--) h->freq[k]=0;
   if (h->type == HISTOGRAM_VARIABLE) h->Min=h->Max=x;
  }

 if (h->type == HISTOGRAM_FIXED)
   AddObservationFixed(x,&(h->low),h->freq,&(h->high),h->Min,h->Max,h->intervals);
  else
   AddObservationVariable(x,h->freq,&(h->Min),&(h->Max),h->intervals);

 h->sample_size++;
}

PRECISION ScalarPrecentile(PRECISION precentile, TScalarHistogram *h)
{
 return Precentile(precentile,h->low,h->freq,h->Min,h->Max,h->intervals,h->sample_size);
}

/*
   Returns the probability that an observation is less than or equal to
   level.

   Assumes
     Let

                I[k]=(h->min + k*inc, h->min + (k+1)*inc),

     where inc=(h->max - h->min)/h->samples_size.  The distribution
     is uniform on I[k] and

       P(h->min + k*inc < x < h->min + (k+1)*inc) = h->freq[k]/h->sample_size.

     Furthermore,

                 P(x < h->min) = 0 and P(x > h->min) = 0.

     In addition, if h->type == FIXED, then

                   P(x = h->min) = h->low/h->sample_size

     and

                  P(x = h->min) = h->high/h->sample_size.
*/
PRECISION ScalarCumulative(PRECISION level, TScalarHistogram *h)
{
 return Cumulative(level,h->low,h->freq,h->Min,h->Max,h->intervals,h->sample_size);
}

TMatrix PlotScalarHistogramAuto(int bins, TScalarHistogram *h)
{
  return MakeHistogramAuto(h->low,h->freq,h->Min,h->high,h->Max,h->intervals,h->sample_size,bins);
}

TMatrix PlotScalarHistogram(PRECISION min, PRECISION max, int bins, TScalarHistogram *h)
{
  return MakeHistogram(h->low,h->freq,h->Min,h->Max,h->intervals,h->sample_size,min,max,bins);
}

/*******************************************************************************
  The following set of routines create a histogram on the fly.  The type must
  be HISTOGRAM_VARIABLE.
*******************************************************************************/
/*
   Resizes the histogram.  After resizing, it is guaranteed that *min <= x <= *max.
   The type of the histogram must be HISTOGRAM_VARIABLE.
*/
static void Resize(PRECISION x, int *h, PRECISION *min, PRECISION *max, int intervals)
{
 int i, j, k, m;
 if (x > *max)
   if (x - *min >= (PRECISION)intervals*(*max - *min))
     {
      for (i=1; i < intervals; i++)
       {
        h[0]+=h[i];
        h[i]=0;
       }
      *max=x;
     }
    else
     {
      m=(int)ceil((x - *min)/(*max - *min));
      for (i=j=0; i < intervals; j++)
       for(h[j]=h[i++], k=1; (k < m) && (i < intervals); k++)
        h[j]+=h[i++];
      for ( ; j < intervals; j++) h[j]=0;
      *max=*min + m*(*max - *min);
      if (x > *max) *max=x;
     }
  else
   if (x < *min)
    if (*max - x >= (PRECISION)intervals*(*max - *min))
      {
       for (j=intervals-1, i=intervals-2; i >= 0; i--)
        {
         h[j]+=h[i];
         h[i]=0;
        }
       *min=x;
      }
    else
     {
      m=(int)ceil((*max - x)/(*max - *min));
      for (i=j=intervals-1; i >= 0; j--)
       for(h[j]=h[i--], k=1; (k < m) && (i >= 0); k++)
        h[j]+=h[i--];
      for ( ; j >= 0; j--) h[j]=0;
      *min=*max - m*(*max - *min);
      if (x < *min) *min=x;
     }
}

/*
   Adds the first observation to a histogram.  The type of the histogram must
   be HISTOGRAM_VARIABLE.
*/
static void InitialObservationVariable(PRECISION x, int *h, PRECISION *min, PRECISION *max, int intervals)
{
 int i;

 *min=x;
 *max=x;

 for (i=0; i < intervals; i++) h[i]=0;

 h[0]=1;
}

/*
   Adds a observation to the histogram.  The type of the histogram must 
   be HISTOGRAM_VARIABLE.   
*/
static void AddObservationVariable(PRECISION x, int *h, PRECISION *min, PRECISION *max, int intervals)
{
 int i;

 if ((x < *min) || (x > *max)) Resize(x,h,min,max,intervals);

 if (*max > *min)                     
   {
    i=(int)(intervals*(x - *min)/(*max - *min));
    h[(i < intervals) ? i : intervals-1]++;
   }
  else
   h[0]++;
}

/*
   Adds a observation to the histogram.  The type of the histogram must
   be HISTOGRAM_FIXED.
*/
static void AddObservationFixed(PRECISION x, int *low, int *h, int *high, PRECISION min, PRECISION max, int intervals)
{
 PRECISION y=floor(intervals*(x - min)/(max - min));
 if (y < 0)
   (*low)++;
  else
   if (y < intervals)
     h[(int)y]++;
    else
     (*high)++;
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/*
   Returns the level such that the probability of observing an observation
   less than or equal to level is percentile.  If there is a point mass at 
   x, and P(y < x) <= percentile <= P(y <= x), then x is returned.

   Assumes
     Both intervals and sample_size are poitive and low and h[i] are
     non-negative.  Also if 
             
           high = sample_size - (low + h[0] + ... + h[intervals - 1]),

     then high is non-negative.

     If min < max, let inc=(max - min)/intervals and define

                     I[k]=(min + k*inc, min + (k+1)*inc),

     The distribution is uniform on I[k] and

           P(min + k*inc < x < min + (k+1)*inc) = h[k]/sample_size.

     Furthermore, there are point masses at min and max with probability

                   P(x = min) = low/sample_size
     and
                   P(x = max) = high/sample_size.

     If min = max, then there is a single point mass at this point.
*/
PRECISION Precentile(PRECISION percentile, int low, int *h, PRECISION min, PRECISION max, int intervals, int sample_size)
{
 int i;
 percentile=percentile*sample_size - low;
 if (percentile <= 0) return min;
 for (i=0; i < intervals; i++)
  if (h[i] && (percentile-=h[i]) <= 0)
   return min + ((PRECISION)(i+1) + percentile/(PRECISION)h[i])*(max - min)/(PRECISION)intervals;
 return max;
}

/*
   Returns the probability that an observation is less than or equal to
   level.

    Assumes
      Both intervals and sample_size are poitive and low and h[i] are
      non-negative.  Also, if 
               
            high = sample_size - (low + h[0] + ... + h[intervals - 1]),

      then high is non-negative.

      If min < max, let inc=(max - min)/intervals and define

                      I[k]=(min + k*inc, min + (k+1)*inc),

      The distribution is uniform on I[k] and

            P(min + k*inc < x < min + (k+1)*inc) = h[k]/sample_size.

      Furthermore, there are point masses at min and max with probability

                    P(x = min) = low/sample_size
      and
                    P(x = max) = high/sample_size.

      If min = max, then there is a single point mass at this point
*/
PRECISION Cumulative(PRECISION level, int low, int *h, PRECISION min, PRECISION max, int intervals, int sample_size)
{
 PRECISION inc=(max-min)/(PRECISION)intervals;
 int i, count;

 if (level < min) return 0.0;
 if (level >= max) return 1.0;

 for (count=low, i=0; i < intervals; count+=h[i++])
  if ((min+=inc) >= level) 
   return ((PRECISION)count + (PRECISION)h[i]*(level - min + inc)/inc)/(PRECISION)sample_size;
 return 1.0;
}

/*
    Returns a histogram over the interval I=[min_out,max_out].  The matrix returned
    has bins rows and 2 columns.  If inc=(max_out - min_out)/bins, then the first 
    element of the ith row is 

                                 min + (i + 0.5)*inc, 
    
    which is the mid-point of the ith interval.  The second element is

                        P(min + i*inc < x <= min + (i + 1)*inc)/inc,

    which is the average density over the ith interval.

    Assumes
     Both intervals and sample_size are poitive and low and h[i] are
     non-negative.  Also if 
             
           high = sample_size - (low + h[0] + ... + h[intervals - 1]),

     then high is non-negative. 

     If min < max, let inc=(max - min)/intervals and define

                     I[k]=(min + k*inc, min + (k+1)*inc),

     The distribution is uniform on I[k] and

           P(min + k*inc < x < min + (k+1)*inc) = h[k]/sample_size.

     Furthermore, there are point masses at min and max with probability

                   P(x = min) = low/sample_size
     and
                   P(x = max) = high/sample_size.

     If min = max, then there is a single point mass at this point.
*/
TMatrix MakeHistogram(int low, int *h, PRECISION min, PRECISION max,
                      int intervals, int sample_size, PRECISION min_out, PRECISION max_out, int bins)
{
 int i;
 PRECISION inc, x, cdf_lower, cdf_upper;
 TMatrix X;

 inc=(max_out-min_out)/(PRECISION)bins;

 if (inc > 0)
   {
    X=CreateMatrix(bins,2);
    x=min_out+inc;

    cdf_lower=Cumulative(min_out,low,h,min,max,intervals,sample_size);

    for (i=0; i < bins; i++)
     {
      cdf_upper=Cumulative(x,low,h,min,max,intervals,sample_size);

      X[i][0]=x - 0.5*inc;
      X[i][1]=(cdf_upper-cdf_lower)/inc;

      cdf_lower=cdf_upper;
      x+=inc;
     }
   }
 else
   return (TMatrix)NULL;

 return X;
}

/*
    Automatically chooses lenth of interval over which to produce histogram and
    then calls MakeHistogram().
*/
TMatrix MakeHistogramAuto(int low, int *h, int high, PRECISION min, PRECISION max, int intervals, int sample_size, int bins)
{
 PRECISION inc=(max-min)/intervals, max_out, min_out;
 int lo, hi;

 if ((low == sample_size) || (inc <= 0))
   {
     min_out=min-1.0;
     max_out=min+1.0;
   }
 else
   {
     if (low > 0)
       lo=-1;
     else
       for (lo=0; (lo < intervals) && !h[lo]; lo++);

     if (lo == intervals)
       {
	 min_out=max-1.0;
	 max_out=max+1.0;
       }
     else
       {
	 if (high > 0)
	   hi=intervals;
	 else
	   for (hi=intervals-1; !h[hi]; hi--);

	 if (lo >= 0)
	   if (hi < intervals)
	     {
	       min_out=min+lo*inc;
	       max_out=min+(hi+1)*inc;
	     }
	   else
	     {
	       min_out=min+lo*inc;
	       if (bins == 1)
		 max_out=(1+SQRT_MACHINE_EPSILON)*max;
	       else
		 {
		   inc=(1-SQRT_MACHINE_EPSILON)*(max - min_out)/(PRECISION)(bins-1);
		   max_out=max + inc;
		 }
	     }
	 else
	   if (hi < intervals)
	     {
	       max_out=min+(hi+1)*inc;
	       if (bins == 1)
		 min_out=(1-SQRT_MACHINE_EPSILON)*min;
	       else
		 {
		   inc=(1-SQRT_MACHINE_EPSILON)*(max_out - min)/(PRECISION)(bins-1);
		   min_out=min - inc;
		 }
	     }
	   else
	     if (bins <= 2)
	       {
		 min_out=(1-SQRT_MACHINE_EPSILON)*min;
		 max_out=(1+SQRT_MACHINE_EPSILON)*max;
	       }
	     else
	       {
		 inc=(1-SQRT_MACHINE_EPSILON)*(max_out - min)/(PRECISION)(bins-2);
		 min_out=min - inc;
		 max_out=max +inc;
	       }
       }
   }
                                     
 return MakeHistogram(low,h,min,max,intervals,sample_size,min_out,max_out,bins);
}


