
#include "m_rand.h"

/******************************************************************************/
/************************ Random Matrices and Vectors *************************/
/******************************************************************************/
/*
   Assumes
     x : m-vector

   Results
     Fills x with deviates drawn from the uniform distribution on [0,1]
*/
TVector UniformVector(TVector x)
{
 int m=V_DIM(x);
 while (--m >= 0) x[m]=uniform();
 return x;
}

/*
   Assumes
     X : m x n matrix

   Results
     Fills X with deviates drawn from the uniform distribution on [0,1]
*/
TMatrix UniformMatrix(TMatrix X)
{
 int i;
 PRECISION *pX;
 if (!X) { MatrixError(NULL_ERR); return (TMatrix)NULL; } 
 for (pX=X[0], i=M_ROW(X)*M_COL(X)-1; i >= 0; i--) pX[i]=uniform();
 return X;
}

/*
   Assumes
     x : m-vector

   Results
     Fills x with standard normal deviates
*/
TVector NormalVector(TVector x)
{
 int i;
 if (!x) { MatrixError(NULL_ERR); return (TVector)NULL; }
 for (i=V_DIM(x)-1; i >= 0; i--) x[i]=gaussian();
 return x;
}

/*
   Assumes
     X : m x n matrix

   Results
     Fills X with standard normal deviates
*/
TMatrix NormalMatrix(TMatrix X)
{
 int j, m=M_ROW(X), n=M_COL(X)-1;
 while (--m >= 0)
  for (j=n; j >= 0; j--) X[m][j]=gaussian();
 return X;
}

/*
   Assumes
     X : m x m matrix
     S : m x m non-singular matrix

   Results
     X is drawn from the Wishart distribution with parameters sigma, nu, and m,
     where sigma=Inverse(S'S).  The pdf of X is proportional to

       |det(X)|^(0.5*(nu - m - 1))
      ---------------------------- * exp(-0.5*tr(Inverse(sigma*X))
         |det(sigma)|^(0.5*nu)


        = |det(X)|^(0.5*(nu - m - 1)) |det(S)|^(0.5*nu) exp(-0.5*tr(S'*X*S))
*/
TMatrix Wishart(TMatrix X, TMatrix S, int nu)
{
 int m=M_ROW(S);
 TMatrix Z;

#ifdef CHECK_SIZE
 if ((m != M_COL(S)) || (m != M_ROW(X)) || (m != M_COL(X))) MatrixError(SIZE_ERR);
#endif

 Z=NormalMatrix(CreateMatrix(m,nu));
 ProductMM(Z,S,Z);
 ProductTransposeMM(X,Z,Z);
 FreeMatrix(Z);
 return X;
}

TMatrix InvertedWishart(TMatrix X, TMatrix S, int nu)
{
 return (TMatrix)NULL;
}

/*
   Assumes
     x : n matrix
     T : n x n upper triangular matrix

   Results
     x is drawn from the multivariate student-t distribution with parameters.
     The pdf of x is given by



*/
void StudentT(TVector x, TMatrix T, int nu)
{
 PRECISION r=0.0, s;
 int i, n=V_DIM(x);
#ifdef CHECK_SIZE
 if ((n != M_COL(T)) || (n != M_ROW(T))) MatrixError(SIZE_ERR);
#endif
 NormalVector(x);
 ProductMV(x,T,x);
 for (i=nu; i > 0; i--)
  {
   s=gaussian();
   r+=s*s;
  }
 ProductSV(x,sqrt((PRECISION)nu/r),x);
}
/******************************************************************************/
/******************************************************************************/

