
/*******************************************************************************

MODEL:

This file contains routines for simulation of vector autoregressions.  The
model is:

   y'[t]*A = x'[t]*F + epsilon'[t]

where
   y[t]       - n_var x 1
   x[t]       - n_pre x 1 = (n_var * n_lag + n_exg) x 1
   epsilon[t] - n_var x 1
   A          - n_var x n_var
   F          - n_pre x n_var

The vector x'(t) = [y'(t-1) ... y'(t-p) z'(t)] contains lagged values of y and
an exogeneous vector z(t), which is (n_exg x 1).

epsilon(t) is a standard normal random variable, independent of x(t).

n_var must be positive.  n_lag, n_exg, and n_pre are non-negative.  If n_lag,
n_exg, and n_pre are all zero, then the model is:

   y'[t]*A = epsilon'[t]


RESTRICTIONS:
Let a[i] be the (n_var x 1) vector which is the the ith column of A and f[i]
be the (n_pre x 1) vector which is the ith column of F.  The vector a[i] is 
assumed to be of the form

                              a[i] = U[i]*b[i]

where U[i] is an (n_var x q[i]) matrix whose columns are orthonormal.  The
vector f[i] is assumed to be for the form

                        f[i] = V[i]*g[i] - W[i]*U[i]*b[i]

where V[i] is an (n_pre x r[i]) matrix whose columns are orthonormal and W[i]
is an (n_pre x n_var) matrix.

This is equivalent to a (? x (n_var + n_pre)) matrix Q[i] such that 

                                     --    --
                                     | a[i] |
                                Q[i]*|      | = 0
                                     | f[i] |
                                     --    --

POSTERIOR DENSITY
The posterior density function is proportional to

   (|det([U[1]*b[1] ... U[n]*b[n]])|^n_obs)*exp(-0.5*b'S b)
                                               *exp(-0.5*(g - P*b)'H (g - P*b))

where

     S  = diag(S[0], ... ,S[n_var-1]) and S[i] is a (q[i] x q[i]) symmetric
            and positive definite matrix
     H  = diag(H[0], ... ,H[n_var-1]) and H[i] is a (r[i] x r[i]) symmetric 
            and positive definite matrix
     P  = diag(P[0], ... ,P[n_var-1]) and  P[i] is a (r[i] x q[i]) matrix
     b' = [b'[0] ... b'[n_var-1]]
     g' = [g'[0] ... g'[n_var-1]]

S[i], H[i], and P[i] satisfy

  S[i] = U'[i]*(S_hat + Z'[i]*(Inverse(H_hat[i]) - V[i]*Inverse(H[i])*V'[i])*Z[i])*U[i]
  H[i] = V'[i]*H_hat[i]*V[i]
  P[i] = Inverse(H[i])*V'[i]*Z[i]*U[i]

where 

  Z[i] = H_hat[i]*(P_hat[i] - W[i])
  S_hat[i] is a (n_var x n_var) symmetric and positive definite matrix
  H_hat[i] is a (n_pre x n_pre) symmetric and positive definite matrix
  P_hat[i] is a (n_pre x n_var) matrix

SIMULATION AND NOTATION
The following matrices are stored for simulation purposes:

    SimulateS[i] = U[i]*Sqrt(Inverse(S[i])) 
    SimulateH[i] = V[i]*Sqrt(Inverse(H[i])) 
    SimulateP[i] = V[i]*H[i]*V'[i]*Z[i] - W[i]

The following matrix is stored for computing log likelihood function:

    LikelihoodS[i] = Sqrt(S[i])*U'[i]

The following matrices are stored for Baysian updating:

    UpdateS[i] = S_hat[i] + P_hat'*H_hat*P_hat
    UpdateH[i] = H_hat[i]
    UpdateP[i] = H_hat[i]*P_hat

The restriction matrices are also stored:

    U[i]
    V[i]
    W[i]

The matrix arrays SimulateH, SimulateP, UpdateH, UpdateP, V, and W will 
be null pointers if n_pre = 0.  

If A is a symmetric positive semi-definite matrix, then we define Sqrt(A) 
to be any matrix B such that B*B'=A.  Note this differs from the standard 
definition.

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

#include "var.h"
#include "exp.h"
#include "m_array.h"
#include "matrix.h"

#include <ctype.h>
#include <string.h>

/* Utilities */
void GetNullVector(TVector v, TMatrix X, int j);
TMatrix GetNullSpace(TMatrix X, int n);
TMatrix MakeSimulationMatrix(TMatrix X, TMatrix Y);
void ClearSimulations(T_SVAR *svar);
TMatrix SymmetricForm(TMatrix X, TMatrix Y);
int Symmetric(TMatrix X);
int OrthonormalColumns(TMatrix X);

/* Compiler directives */
//#define USE_SINGULAR_VALUE_DECOMPOSITION
//#define VERBOSE

/******************************************************************************/
/****************************** Error Functions *******************************/
/******************************************************************************/
static int SVAR_ERROR=0;
int Set_SVAR_Error(int err)
{ return (SVAR_ERROR=err); }
int Get_SVAR_Error(void)
{ return SVAR_ERROR; }
char *Get_SVAR_Error_String(void)
{
 switch (SVAR_ERROR)
  {
   case SVAR_NO_ERR: return "No error";
   case SVAR_SIZE_ERR: return "Vector, matrix or matrix array not correct size"; 
   case SVAR_PARAMETER_ERR: return "Parameter not in correct range";
   case SVAR_MEMORY_ERR: return "Out of memory";
   case SVAR_SYMMETRIC_ERR: return "Matrix not symmetric";
   case SVAR_ORTHONORMAL_ERR: return "Columns of matrix not orthonormal";
   case SVAR_RESTRICTION_ERR: return "Too many contemporaneous restrictions";
   case SVAR_POSDEF_ERR: return "Matrix not positive definite";
   case SVAR_SIMULATE_ERR: return "T_SVAR structure not ready for simulation";
   case SVAR_DECOMPOSITION_ERR: return "Error in decomposition routine";
   default: return "Unknown error";
  }
}

/******************************************************************************/
/************************ Constructor and Distructors *************************/
/******************************************************************************/
/*
   Assumes
     n_var : number of endogenous variables
     n_pre : number of predetermined variables
     n_lag : number of lags

   Results
     Makes T_SVAR data structure.

   Notes
     In a properly constructed T_SVAR structure, the following are guaranteed:
       
       n_var >  0
       n_lag >= 0
       n_exg >= 0
       n_pre == n_var*n_lag + n_exg
       n_obs >= 0

       U           : Matrix array of length n_var.

       V           : Matrix array of length n_var if n_pre > 0 and null otherwise.

       W           : Matrix array of length n_var of (n_pre x n_var) matrices 
                      if n_pre > 0 and null otherwise.

       UpdateS     : Matrix array of length n_var of (n_var x n_var) symmetric
                      matrices.

       UpdateH     : Matrix array of length n_var of (n_pre x n_pre) symmetric
                      matrices if n_pre > 0 and null otherwise.

       UpdateP     : Matrix array of length n_var of (n_pre x n_var) matrices
                      if n_pre > 0 and null otherwise.

       SimulateS   : Matrix array of length n_var.

       SimulateH   : Matrix array of length n_var if n_pre > 0 and null otherwise.

       SimulateP   : Matrix array of length n_var if n_pre > 0 and null otherwise.
                        
     Construct_SVAR only returns a properly constructed T_SVAR structure. 
*/
T_SVAR* Construct_SVAR(int n_var, int n_lag, int n_exg)
{
 T_SVAR *svar;
 int i;

 if ((n_var <= 0) || (n_lag < 0) || (n_exg < 0))
  {
   Set_SVAR_Error(SVAR_PARAMETER_ERR);
   return (T_SVAR*)NULL;
  }
  
 if ((svar=(T_SVAR*)malloc(sizeof(T_SVAR))) == (T_SVAR*)NULL) 
  {
   Set_SVAR_Error(SVAR_MEMORY_ERR);
   return (T_SVAR*)NULL;
  }

 svar->n_var=n_var;
 svar->n_pre=n_var*n_lag+n_exg;
 svar->n_exg=n_exg;
 svar->n_lag=n_lag;

 svar->n_obs=0;

 svar->U=(TMatrix*)NULL;
 svar->V=(TMatrix*)NULL;
 svar->W=(TMatrix*)NULL;

 svar->UpdateS=(TMatrix*)NULL;
 svar->UpdateH=(TMatrix*)NULL;
 svar->UpdateP=(TMatrix*)NULL;
   
 svar->SimulateS=(TMatrix*)NULL;
 svar->SimulateH=(TMatrix*)NULL;
 svar->SimulateP=(TMatrix*)NULL;

 if (!(svar->U=CreateMatrixArray(n_var)) ||
     !(svar->UpdateS=CreateMatrixArray(n_var)) ||
     !(svar->SimulateS=CreateMatrixArray(n_var)))
  {
   Set_SVAR_Error(SVAR_MEMORY_ERR);
   Destruct_SVAR(svar);
   return (T_SVAR*)NULL;
  }
 for (i=n_var-1; i >= 0; i--)
  if (!(svar->UpdateS[i]=InitializeMatrix(CreateMatrix(n_var,n_var),0.0)) ||
      !(svar->U[i]=IdentityMatrix((TMatrix)NULL,n_var)))
   {
    Set_SVAR_Error(SVAR_MEMORY_ERR);
    Destruct_SVAR(svar);
    return (T_SVAR*)NULL;
   }
  
 if (svar->n_pre > 0)
  {
   if (!(svar->V=CreateMatrixArray(n_var)) || 
       !(svar->W=CreateMatrixArray(n_var)) ||
       !(svar->UpdateH=CreateMatrixArray(n_var)) || 
       !(svar->UpdateP=CreateMatrixArray(n_var)) ||       
       !(svar->SimulateH=CreateMatrixArray(n_var)) ||
       !(svar->SimulateP=CreateMatrixArray(n_var)))
    {
     Set_SVAR_Error(SVAR_MEMORY_ERR);
     Destruct_SVAR(svar);
     return (T_SVAR*)NULL;
    }
   for (i=n_var-1; i >= 0; i--)
    if (!(svar->UpdateH[i]=InitializeMatrix(CreateMatrix(svar->n_pre,svar->n_pre),0.0)) || 
        !(svar->UpdateP[i]=InitializeMatrix(CreateMatrix(svar->n_pre,n_var),0.0)) || 
        !(svar->V[i]=IdentityMatrix((TMatrix)NULL,svar->n_pre)) || 
        !(svar->W[i]=InitializeMatrix(CreateMatrix(svar->n_pre,n_var),0.0)))
     {
      Set_SVAR_Error(SVAR_MEMORY_ERR);
      Destruct_SVAR(svar);
      return (T_SVAR*)NULL;
     }
  }

 return svar;
}

/*
   Frees memory allocated for T_SVAR structure.
*/
void Destruct_SVAR(T_SVAR *svar)
{
 if (svar)
  {
   FreeMatrixArray(svar->U);
   FreeMatrixArray(svar->V);
   FreeMatrixArray(svar->W);
   FreeMatrixArray(svar->UpdateS);
   FreeMatrixArray(svar->UpdateH);
   FreeMatrixArray(svar->UpdateP);
   FreeMatrixArray(svar->SimulateS);
   FreeMatrixArray(svar->SimulateH);
   FreeMatrixArray(svar->SimulateP);
   free(svar);
  }
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/******************************************************************************/
/*************************** Restriction Functions ****************************/
/******************************************************************************/
/*
  Returns
   Zero upon success and non-zero error code otherwise.

  Results
   The matrices svar->U, svar->V, and svar->W are set so that no restrictions
   are imposed.  Calls SetSimulations().  Restrictions are unchanged upon
   failure.
*/
int NoRestrictions(T_SVAR *svar)
{
 int i;
 TMatrix *U=(TMatrix*)NULL, *V=(TMatrix*)NULL;

 Set_SVAR_Error(SVAR_NO_ERR);
 if (!(U=CreateMatrixArray(svar->n_var))) 
   Set_SVAR_Error(SVAR_MEMORY_ERR);
  else
   for (i=svar->n_var-1; i >= 0; i--)
    if (!(U[i]=IdentityMatrix((TMatrix)NULL,svar->n_var))) 
     Set_SVAR_Error(SVAR_MEMORY_ERR);

 if (svar->n_pre > 0)
  if (!(V=CreateMatrixArray(svar->n_var))) 
    Set_SVAR_Error(SVAR_MEMORY_ERR);
   else
    for (i=svar->n_var-1; i >= 0; i--)
     if (!(V[i]=IdentityMatrix((TMatrix)NULL,svar->n_pre))) 
      Set_SVAR_Error(SVAR_MEMORY_ERR);      
  
 if (Get_SVAR_Error())
  {
   FreeMatrixArray(U);
   FreeMatrixArray(V);
   return Get_SVAR_Error();
  }
 
 FreeMatrixArray(svar->U);
 svar->U=U;
 if (svar->n_pre > 0)
  {
   FreeMatrixArray(svar->V);
   svar->V=V;
   for (i=svar->n_var-1; i >= 0; i--) InitializeMatrix(svar->W[i],0.0);
  }
 SetSimulations(svar);
 return SVAR_NO_ERR; 
}

/*
  Assumes
   Q - matrix array of length n_var of (? x (n_var + n_pre)) matrices.
   
  Returns
   Zero upon success and non-zero error code otherwise.

  Results
   Set the restrictions using the passed linear restrictions and calls
   SetSimulations().  Restrictions are unchanged upon failure.

  Notes
   The restrictions imposed are 
   
                            --    --
                            | a[i] |
                     Q[i] * |      | = 0
                            | f[i] |
                            --    --
   
   If Q[i] is the null pointer, then no restrictions are imposed on
   that column.                       
*/
int SetLinearRestrictions(TMatrix *Q, T_SVAR *svar)
{
 TMatrix *U=(TMatrix*)NULL, *V=(TMatrix*)NULL, *W=(TMatrix*)NULL;
 TMatrix U_tilda, V_tilda, V_hat, W_hat, A, B, C, E;
 TVector D;
 PRECISION max;
 int i, j, k, r, s, a, b;

 for (i=svar->n_var-1; i >= 0; i--)
  if (Q[i] && (M_COL(Q[i]) != svar->n_var + svar->n_pre))
   return Set_SVAR_Error(SVAR_SIZE_ERR);
    
 Set_SVAR_Error(SVAR_NO_ERR);
 if (!(U=CreateMatrixArray(svar->n_var))) 
   Set_SVAR_Error(SVAR_MEMORY_ERR);
  else
   if (svar->n_pre > 0)
     if (!(V=CreateMatrixArray(svar->n_var)) || !(W=CreateMatrixArray(svar->n_var))) 
       Set_SVAR_Error(SVAR_MEMORY_ERR);
      else
       for (i=svar->n_var-1; i >= 0; i--)
        if (!Q[i]) 
          {
           if (!(U[i]=IdentityMatrix((TMatrix)NULL,svar->n_var)) 
                   || !(V[i]=IdentityMatrix((TMatrix)NULL,svar->n_pre)) 
                          || !(W[i]=InitializeMatrix(CreateMatrix(svar->n_pre,svar->n_var),0.0)))
            Set_SVAR_Error(SVAR_MEMORY_ERR);
          }
         else
          {
           V_tilda=SubMatrix((TMatrix)NULL,Q[i],0,svar->n_var,M_ROW(Q[i]),svar->n_pre);
           D=CreateVector(svar->n_pre);
           A=CreateMatrix(svar->n_pre,svar->n_pre);              
           if (!SVD(V_tilda,D,A,V_tilda))
             Set_SVAR_Error(SVAR_DECOMPOSITION_ERR);
            else
             {
              for (max=D[0], j=svar->n_pre-1; j > 0; j--) 
               if (max < fabs(D[j])) max=fabs(D[j]);
              max*=SQRT_MACHINE_EPSILON;
              for (r=0, j=svar->n_pre-1; j >= 0; j--)
               if (fabs(D[j]) <= max) r++;
              s=svar->n_pre - r;

              if (r > 0) V[i]=CreateMatrix(svar->n_pre,r);
              if (s > 0)
                {
                 V_hat=CreateMatrix(svar->n_pre,s);
                 B=CreateMatrix(M_ROW(Q[i]),s);
                }
               else
                {
                 V_hat=(TMatrix)NULL;
                 B=(TMatrix)NULL;
                }
              C=CreateMatrix(M_ROW(Q[i]),M_ROW(Q[i]));
              E=SubMatrix((TMatrix)NULL,Q[i],0,0,M_ROW(Q[i]),svar->n_var);

              if (!C || !E || ((s > 0) && (!V_hat || !B)) || ((r > 0) && !V[i]))
                Set_SVAR_Error(SVAR_MEMORY_ERR);
               else
                {
                 for (a=r, b=s, j=svar->n_pre-1; j >= 0; j--)
                  if (fabs(D[j]) <= max)
                    for (a--, k=svar->n_pre-1; k >= 0; k--) V[i][k][a]=A[k][j];
                   else
                    {
                     for (b--, k=svar->n_pre-1; k >= 0; k--) V_hat[k][b]=A[k][j];
                     for (k=M_ROW(Q[i])-1; k >= 0; k--) B[k][b]=D[j]*V_tilda[k][j];
                    }                       
                 if (!QR(C,B,B))
                   Set_SVAR_Error(SVAR_DECOMPOSITION_ERR);
                  else   
                   {
                    TransposeProductMM(E,C,E);
                    for (k=M_ROW(B)-1; k >= 0; k--)
                     {
                      for (j=M_COL(C)-1; j >= M_COL(B); j--) C[k][j]=0.0;
                      for ( ; j >= 0; j--) C[k][j]=B[k][j];
                     }
                    for (k=M_COL(C)-1; k >= M_COL(B); k--) C[k][k]=1.0; 
                    if (!InverseProductUM(E,C,E))
                      Set_SVAR_Error(SVAR_DECOMPOSITION_ERR);
                     else
                      {
                       if (M_ROW(Q[i])-s > 0)
                         {
                          U[i]=GetNullSpace(U_tilda=SubMatrix((TMatrix)NULL,E,s,0,M_ROW(Q[i])-s,svar->n_var),svar->n_var);                        
                          FreeMatrix(U_tilda);
                         }
                        else
                         U[i]=IdentityMatrix((TMatrix)NULL,svar->n_var);
                       if (s > 0)
                         {
                          W[i]=ProductMM((TMatrix)NULL,V_hat,W_hat=SubMatrix((TMatrix)NULL,E,0,0,s,svar->n_var));
                          FreeMatrix(W_hat);
                         }
                        else
                         W[i]=InitializeMatrix(CreateMatrix(svar->n_pre,svar->n_var),0.0);
                       if (!U[i] || !W[i]) Set_SVAR_Error(SVAR_MEMORY_ERR);
                      }
                   }
                 FreeMatrix(V_hat);
                 FreeMatrix(B);
                 FreeMatrix(C);
                 FreeMatrix(E);
                }
             }
           FreeMatrix(A);
           FreeVector(D);
           FreeMatrix(V_tilda);
          } 
    else
     for (i=svar->n_var-1; i >= 0; i--)
      U[i]=GetNullSpace(Q[i],svar->n_var);
        
 if (Get_SVAR_Error())
  {
   FreeMatrixArray(U);
   FreeMatrixArray(V);
   FreeMatrixArray(W);
   return Get_SVAR_Error();
  }
 
 FreeMatrixArray(svar->U);
 svar->U=U;
 if (svar->n_pre > 0)
  {
   FreeMatrixArray(svar->V);
   svar->V=V;
   FreeMatrixArray(svar->W);
   svar->W=W;
  }
 SetSimulations(svar);
 return SVAR_NO_ERR; 
}

/*
  Assumes
   EC - (n_var x n_var) contemporaneous exclusion matrix or null pointer
   EP - (n_pre x n_var) predetermined exclusion matrix or null pointer

  Returns
   Zero upon success and non-zero error code otherwise.

  Results
   Set the restrictions using the passed exclusion matrices.  By convention,
   passing a null matrix is equivalent to passing a matrix of ones.  Calls
   SetSimulations().  Restrictions are unchanged upon failure.

  Notes
   If an element on the exclusion matrix is non-zero, then the corresponding
   coefficient is unrestricted.  If an element is zero, then the coefficient
   is restricted to be zero.
*/
int SetExclusionRestrictions(TMatrix EC, TMatrix EP, T_SVAR *svar)
{
 int i, j, q;

 TMatrix *U=(TMatrix*)NULL, *V=(TMatrix*)NULL;

 Set_SVAR_Error(SVAR_NO_ERR);
 if (!(U=CreateMatrixArray(svar->n_var))) 
   Set_SVAR_Error(SVAR_MEMORY_ERR);
  else
   {
    if (!EC)
     for (j=svar->n_var; j >= 0; j--)
       {
        if (!(U[j]=IdentityMatrix((TMatrix)NULL,svar->n_var))) 
         Set_SVAR_Error(SVAR_MEMORY_ERR); 
       }
      else
       if ((M_ROW(EC) != svar->n_var) || (M_COL(EC) != svar->n_var))
         Set_SVAR_Error(SVAR_SIZE_ERR);
        else
         for (j=svar->n_var-1; j >= 0; j--)
          {
           for (q=0, i=svar->n_var-1; i >= 0; i--)
            if (fabs(EC[i][j]) > SQRT_MACHINE_EPSILON) q++;
           if (!q) 
             Set_SVAR_Error(SVAR_RESTRICTION_ERR);
            else
             if (!(U[j]=InitializeMatrix(CreateMatrix(svar->n_var,q),0.0)))
               Set_SVAR_Error(SVAR_MEMORY_ERR);
              else
               for (i=svar->n_var-1; i >= 0; i--)
                if (fabs(EC[i][j]) > SQRT_MACHINE_EPSILON) U[j][i][--q]=1.0;
          }

    if (svar->n_pre > 0)
      if (!(V=CreateMatrixArray(svar->n_var)))
        Set_SVAR_Error(SVAR_MEMORY_ERR);
       else
        if (!EP)
          for (j=svar->n_var-1; j >= 0; j--)
           {
            if (!(V[j]=IdentityMatrix((TMatrix)NULL,svar->n_pre)))
              Set_SVAR_Error(SVAR_MEMORY_ERR);
           }
         else
          if ((M_ROW(EP) != svar->n_pre) || (M_COL(EP) != svar->n_var))
            Set_SVAR_Error(SVAR_SIZE_ERR);
           else
            for (j=svar->n_var-1; j >= 0; j--)
             {
              for (q=0, i=svar->n_pre-1; i >= 0; i--)
               if (fabs(EP[i][j]) > SQRT_MACHINE_EPSILON) q++;        
              if (q > 0)
               if (!(V[j]=InitializeMatrix(CreateMatrix(svar->n_pre,q),0.0)))
                 Set_SVAR_Error(SVAR_MEMORY_ERR);
                else
                 for (i=svar->n_pre-1; i >= 0; i--)
                  if (fabs(EP[i][j]) > SQRT_MACHINE_EPSILON) V[j][i][--q]=1.0;
             }     
     else
      if (EP) Set_SVAR_Error(SVAR_SIZE_ERR);
   }

 if (Get_SVAR_Error())
  {
   FreeMatrixArray(U);
   FreeMatrixArray(V);
   return Get_SVAR_Error();
  }
  
 FreeMatrixArray(svar->U);
 svar->U=U;
 if (svar->n_pre > 0)
  {
   FreeMatrixArray(svar->V);
   svar->V=V;
   for (j=svar->n_var-1; j >= 0; j--) InitializeMatrix(svar->W[j],0.0);
  }
 SetSimulations(svar);
 return SVAR_NO_ERR;
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/******************************************************************************/
/********************************** Updating **********************************/
/******************************************************************************/
/*
   Assumes
     YY   : (n_var x n_var) symmetric and positive semi-definite matrices.
     XX   : (n_pre x n_pre) symmetric and positive semi-definite matrices, or null pointer.
     XY   : (n_pre x n_var) matrices, or null pointer.
     n_obs: number of observations

   Returns
     Zero upon success and non-zero error code otherwise.

   Results
     Performs an Baysian update of the T_SVAR data structure.

        svar->UpdateH[i] = svar->UpdateH[i] + XX
        svar->UpdateP[i] = svar->UpdateP[i] + XY
        svar->UpdateS[i] = svar->UpdateS[i] + YY

     If unable to perform the update, the T_SVAR structure is unchanged.  Calls
     SetSimulations().
*/
int BaysianUpdate(TMatrix YY, TMatrix XX, TMatrix XY, int n_obs, T_SVAR *svar)
{
 int i; 
 if (n_obs < 0) return Set_SVAR_Error(SVAR_PARAMETER_ERR);
 if (CheckUpdateMatrices(YY,XX,XY,svar->n_var,svar->n_pre)) return Get_SVAR_Error();   
 svar->n_obs+=n_obs;
 for (i=svar->n_var-1; i >= 0; i--)
  AddMM(svar->UpdateS[i],svar->UpdateS[i],YY);
 if (svar->n_pre > 0)
  for (i=svar->n_var-1; i >= 0; i--)
   {
    AddMM(svar->UpdateH[i],svar->UpdateH[i],XX);
    AddMM(svar->UpdateP[i],svar->UpdateP[i],XY);
   }
 SetSimulations(svar);
 return SVAR_NO_ERR;
}

/* 
   Assumes
    Y     : data matrix with n_var columns.
    Z     : data matrix with n_exg columns or null pointer.
    y1    : first observation in data matrix Y. 
    z1    : first observation in data matrix Z.
    T     : number of observations to include.  If T is not positive, then T
            is set to the maximum number of observations possible.
    n_obs : number of observations to pass to HomogenousUpdate().  If n_obs
            is negative, then n_obs is set to T.

   Returns
    Zero upon success and non-zero error code otherwise.

   Results
    Performs Baysian updating of T_SVAR structure using given data.  If

              Y[y1]                Y[y1-1]  ...   Y[y1-p]     Z[z1]
                .                     .              .          .
        Y =     .             X =     .              .          .
                .                     .              .          .
            Y[y1+T-1]             Y[y1+T-2] ... Y[y1+T-1-p] Z[z1+T-1]

    where Y[i] is the ith row of Y if i >= 0 and is the zero vector if i < 0,
    then updating is performed by calling BaysianUpdate(Y'Y,X'X,Y'X,n_obs).
    
    If unable to perform the update, the T_SVAR structure is unchanged.  Calls
    SetSimulations().      
*/
int DataUpdate(TMatrix Y, TMatrix Z, int y1, int z1, int T, int n_obs, T_SVAR *svar)
{
 int i, j, k, rtrn;
 PRECISION tmp;
 TMatrix X, YY, XX, XY;

 if (!Y || (M_COL(Y) != svar->n_var)) return Set_SVAR_Error(SVAR_SIZE_ERR);
 if (svar->n_exg == 0)
   {
    if (Z) return Set_SVAR_Error(SVAR_SIZE_ERR);
    if (T <= 0) T=M_ROW(Y)-y1;
    if ((T <= 0) || (y1 < 0) || (M_ROW(Y) < T+y1))
     return Set_SVAR_Error(SVAR_PARAMETER_ERR);
   }
  else
   {
    if (!Z || (M_COL(Z) != svar->n_exg)) return Set_SVAR_Error(SVAR_SIZE_ERR);
    if (T <= 0) T=((M_ROW(Y)-y1) < (M_ROW(Z)-z1)) ? M_ROW(Y)-y1 : M_ROW(Z)-z1;
    if ((T <= 0) || (y1 < 0) || (z1 < 0) || (M_ROW(Y) < T+y1) || (M_ROW(Z) < T+z1)) 
     return Set_SVAR_Error(SVAR_PARAMETER_ERR);
   }

 if (n_obs < 0) n_obs=T;

 if (!(YY=CreateMatrix(svar->n_var,svar->n_var)))
   rtrn=Set_SVAR_Error(SVAR_MEMORY_ERR);
  else
   {
    for (i=svar->n_var-1; i >= 0; i--)
     for (j=svar->n_var-1; j >= 0; j--)
      {
       tmp=0.0;
       for (k=T+y1-1; k >= y1; k--)
        tmp+=Y[k][i]*Y[k][j];
       YY[i][j]=tmp;     
      }
    if (svar->n_pre > 0)
      if (!(X=CreateMatrix(T,svar->n_pre)))
        rtrn=Set_SVAR_Error(SVAR_MEMORY_ERR);
       else
        {
         for (i=0; i < T; i++)
          {
           for (j=1; j <= svar->n_lag; j++)
            if (i+y1-j >= 0)
              memcpy(X[i]+(j-1)*svar->n_var,Y[i+y1-j],svar->n_var*sizeof(PRECISION));
             else
              for (k=(j-1)*svar->n_var; k < j*svar->n_var; k++) X[i][k]=0.0;
           if (svar->n_exg > 0) memcpy(X[i]+svar->n_lag*svar->n_var,Z[i+z1],svar->n_exg*sizeof(PRECISION)); 
          }
         if (!(XX=TransposeProductMM((TMatrix)NULL,X,X)))
           rtrn=Set_SVAR_Error(SVAR_MEMORY_ERR);
          else
           {
            if (!(XY=CreateMatrix(svar->n_pre,svar->n_var)))
              rtrn=Set_SVAR_Error(SVAR_MEMORY_ERR);
             else
              {
               for (i=svar->n_pre-1; i >= 0; i--)
                for (j=svar->n_var-1; j >= 0; j--)
                 {
                  tmp=0.0;
                  for (k=T-1; k >= 0; k--) tmp+=X[k][i]*Y[k+y1][j];
                  XY[i][j]=tmp;
                 }
               rtrn=BaysianUpdate(YY,XX,XY,n_obs,svar);
               FreeMatrix(XY);
              }
            FreeMatrix(XX);
           }
         FreeMatrix(X);
        }
     else
      rtrn=BaysianUpdate(YY,(TMatrix)NULL,(TMatrix)NULL,n_obs,svar);
    FreeMatrix(YY);
   }
 return rtrn;
}

/*
   Assumes
     S_bar : matrix array of length n_var of (n_var x n_var) symmetric and 
             positive definite matrices.
     H_bar : matrix array of length n_var of (n_pre x n_pre) symmetric and 
             positive definite matrices, or null pointer.
     P_bar : matrix array of length n_var of (n_pre x n_var) matrices, or 
             null pointer.
     n_obs : number of observations

   Results
     Performs an update of the T_SVAR data structure.

      svar->UpdateH[i] = svar->UpdateH[i] + H_bar[i]
      svar->UpdateP[i] = svar->UpdateP[i] + H_bar[i]*P_bar[i]
      svar->UpdateS[i] = svar->UpdateS[i] + S_bar[i] + P_bar[i]'*H_bar[i]*P_bar[i]

     If unable to perform the update, the T_SVAR structure is unchanged.  Calls
     SetSimulations().         
*/
int PriorUpdate(TMatrix *S_bar, TMatrix *H_bar, TMatrix *P_bar, int n_obs, T_SVAR *svar)
{
 int i;
 TMatrix P, S;
 if (n_obs < 0) return Set_SVAR_Error(SVAR_PARAMETER_ERR);
 if (CheckUpdateArrays(S_bar,H_bar,P_bar,svar->n_var,svar->n_pre)) return Get_SVAR_Error();    
 svar->n_obs+=n_obs;
 if (svar->n_pre > 0)
   {
    if (!(P=CreateMatrix(svar->n_pre,svar->n_pre)))
     return Set_SVAR_Error(SVAR_MEMORY_ERR);
    if (!(S=CreateMatrix(svar->n_var,svar->n_var)))
     {
      FreeMatrix(P);
      return Set_SVAR_Error(SVAR_MEMORY_ERR);
     }    
    for (i=svar->n_var-1; i >= 0; i--)
     {
      AddMM(svar->UpdateH[i],svar->UpdateH[i],H_bar[i]);
      AddMM(svar->UpdateP[i],svar->UpdateP[i],ProductMM(P,H_bar[i],P_bar[i]));
      AddMM(svar->UpdateS[i],svar->UpdateS[i],S_bar[i]);
      AddMM(svar->UpdateS[i],svar->UpdateS[i],TransposeProductMM(S,P_bar[i],P));  
     }
    FreeMatrix(S);
    FreeMatrix(P);
   }
  else
   for (i=svar->n_var-1; i >= 0; i--)
    AddMM(svar->UpdateS[i],svar->UpdateS[i],S_bar[i]);
 SetSimulations(svar);
 return SVAR_NO_ERR;
}

/*
   Assumes
     S_bar : (n_var x n_var) symmetric and positive definite matrix.
     H_bar : (n_pre x n_pre) symmetric and positive definite matrix, or null pointer.
     P_bar : (n_pre x n_var) matrix, or null pointer.
     n_obs : number of observations

   Results
     Performs an update of the T_SVAR data structure.

      svar->UpdateH[i] = svar->UpdateH[i] + H_bar
      svar->UpdateP[i] = svar->UpdateP[i] + H_bar*P_bar
      svar->UpdateS[i] = svar->UpdateS[i] + S_bar + P_bar'*H_bar*P_bar

     If unable to perform the update, the T_SVAR structure is unchanged.  Calls
     SetSimulations().         

*/
int HomogenousPriorUpdate(TMatrix S_bar, TMatrix H_bar, TMatrix P_bar, int n_obs, T_SVAR *svar)
{
 int i;
 TMatrix P, S;
 if (n_obs < 0) return Set_SVAR_Error(SVAR_PARAMETER_ERR);
 if (CheckUpdateMatrices(S_bar,H_bar,P_bar,svar->n_var,svar->n_pre)) return Get_SVAR_Error();   
 if (svar->n_pre > 0)
   {   
    if (!(P=ProductMM((TMatrix)NULL,H_bar,P_bar))) 
     return Set_SVAR_Error(SVAR_MEMORY_ERR);
    if (!(S=TransposeProductMM((TMatrix)NULL,P_bar,P)))
     {
      FreeMatrix(P);
      return Set_SVAR_Error(SVAR_MEMORY_ERR);
     }
    AddMM(S,S_bar,S);
    for (i=svar->n_var-1; i >= 0; i--)
     {
      AddMM(svar->UpdateH[i],svar->UpdateH[i],H_bar);
      AddMM(svar->UpdateP[i],svar->UpdateP[i],P);
      AddMM(svar->UpdateS[i],svar->UpdateS[i],S);
     }
    FreeMatrix(P);
    FreeMatrix(S);
   }
  else
   for (i=svar->n_var-1; i >= 0; i--)
    AddMM(svar->UpdateS[i],svar->UpdateS[i],S_bar);
 svar->n_obs+=n_obs;
 SetSimulations(svar);
 return SVAR_NO_ERR;
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
void ClearSimulations(T_SVAR *svar)
{
 int i;
 for (i=svar->n_var-1; i >= 0; i--) 
  {
   FreeMatrix(svar->SimulateS[i]); 
   svar->SimulateS[i]=(TMatrix)NULL;
  }
 if (svar->n_pre > 0)
  for (i=svar->n_var-1; i >= 0; i--)
   {
    FreeMatrix(svar->SimulateH[i]);  
    svar->SimulateH[i]=(TMatrix)NULL;
    FreeMatrix(svar->SimulateP[i]);  
    svar->SimulateP[i]=(TMatrix)NULL;
   }
}

/*
   Sets

     SimulateH[i] = V[i]*Sqrt(Inverse(V'[i]*UpdateH[i]*V[i]))
     SimulateP[i] = A[i]*B[i] - W[i] 
     SimulateS[i] = U[i]*Sqrt(Inverse(U'[i]*D[i]*U[i]))   

   where 
     A[i] = SimulateH[i] * SimulateH'[i]
     B[i] = UpdateP[i] + UpdateH[i] * W[i]
     C[i] = Inverse(UpdateH[i])
     D[i] = UpdateS[i] - UpdateP'[i]*C[i]*UpdateP[i] + B'[i]*(C[i] - A[i])*B[i]

     The matrix Sqrt(X) can be taken to be any matrix such that
     Sqrt(X)*Sqrt(X)' = X.
*/
int SetSimulations(T_SVAR *svar)
{
 int i;
 TMatrix A, B, C, D, E;
 ClearSimulations(svar);
 if (svar->n_pre > 0)
   for (i=svar->n_var-1; i >= 0; i--)
    if (svar->V[i])
      {        
       svar->SimulateH[i]=MakeSimulationMatrix(svar->V[i],svar->UpdateH[i]);
     
       A=ProductTransposeMM((TMatrix)NULL,svar->SimulateH[i],svar->SimulateH[i]);
       B=ProductMM((TMatrix)NULL,svar->UpdateH[i],svar->W[i]);
       AddMM(B,svar->UpdateP[i],B);
       svar->SimulateP[i]=ProductMM((TMatrix)NULL,A,B);
       SubtractMM(svar->SimulateP[i],svar->SimulateP[i],svar->W[i]);

       C=Inverse_Cholesky((TMatrix)NULL,svar->UpdateH[i]);
       D=SymmetricForm(svar->UpdateP[i],C);
       SubtractMM(D,svar->UpdateS[i],D);
       SubtractMM(C,C,A);
       E=SymmetricForm(B,C);
       AddMM(D,D,E);
       svar->SimulateS[i]=MakeSimulationMatrix(svar->U[i],D);

       FreeMatrix(E);
       FreeMatrix(D);
       FreeMatrix(C);
       FreeMatrix(B);
       FreeMatrix(A);

       if (!svar->SimulateS[i] || !svar->SimulateH[i] || !svar->SimulateP[i])
        {
         ClearSimulations(svar);
         return Set_SVAR_Error(SVAR_SIMULATE_ERR);
        }
      }
     else
      {
       B=ProductMM((TMatrix)NULL,svar->UpdateH[i],svar->W[i]);
       AddMM(B,svar->UpdateP[i],B);
       svar->SimulateP[i]=MinusM((TMatrix)NULL,svar->W[i]);

       C=Inverse_Cholesky((TMatrix)NULL,svar->UpdateH[i]);     
       D=SymmetricForm(svar->UpdateP[i],C);
       SubtractMM(D,svar->UpdateS[i],D);
       E=SymmetricForm(B,C);
       AddMM(D,D,E);
       svar->SimulateS[i]=MakeSimulationMatrix(svar->U[i],D);

       FreeMatrix(E);
       FreeMatrix(D);
       FreeMatrix(C);
       FreeMatrix(B);

       if (!svar->SimulateS[i] || !svar->SimulateP[i])
        {
         ClearSimulations(svar);
         return Set_SVAR_Error(SVAR_SIMULATE_ERR);
        }
      }
  else
   for (i=svar->n_var-1; i >= 0; i--)
    if (!(svar->SimulateS[i]=MakeSimulationMatrix(svar->U[i],svar->UpdateS[i])))
     {
      ClearSimulations(svar);
      return Set_SVAR_Error(SVAR_SIMULATE_ERR);
     }  
 return SVAR_NO_ERR;
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/******************************************************************************/
/******************************* Data Checking ********************************/
/******************************************************************************/
/*
   Returns 0 if U, V and W are valid restrictions and non-zero error code 
   otherwise.  Sets SVAR_ERROR.
*/
int CheckRestrictions(TMatrix *U, TMatrix *V, TMatrix *W, int n_var, int n_pre)
{
 int i;
 if (!U || (MA_LEN(U) != n_var)) return Set_SVAR_Error(SVAR_SIZE_ERR);
 for (i=0; i < n_var; i++)
  {
   if (!U[i] || (M_ROW(U[i]) != n_var)) return Set_SVAR_Error(SVAR_SIZE_ERR);
   if (!OrthonormalColumns(U[i])) return Set_SVAR_Error(SVAR_ORTHONORMAL_ERR);
  }
 if (n_pre > 0)
   {
    if (!V || (MA_LEN(V) != n_var) || !W  || (MA_LEN(W) != n_var))
     return Set_SVAR_Error(SVAR_SIZE_ERR);
    for (i=0; i < n_var; i++)
     {
      if (!V[i] || (M_ROW(V[i]) != n_pre)) return Set_SVAR_Error(SVAR_SIZE_ERR);
      if (!OrthonormalColumns(V[i])) return Set_SVAR_Error(SVAR_ORTHONORMAL_ERR);
      if (!W[i] || (M_ROW(W[i]) != n_pre) || (M_COL(W[i]) != n_var))
       return Set_SVAR_Error(SVAR_SIZE_ERR);
     }
   }
  else
   if (V || W) return Set_SVAR_Error(SVAR_SIZE_ERR);
 return SVAR_NO_ERR;
}

/*
   Returns 0 if updates are OK and error code otherwise.  Sets SVAR_ERROR.
*/
int CheckUpdateMatrices(TMatrix UpdateS, TMatrix UpdateH, TMatrix UpdateP, int n_var, int n_pre)
{
 if (!UpdateS || (M_ROW(UpdateS) != n_var) || (M_COL(UpdateS) != n_var)) 
  return Set_SVAR_Error(SVAR_SIZE_ERR);
 if (!Symmetric(UpdateS))
  return Set_SVAR_Error(SVAR_SYMMETRIC_ERR);
 if (n_pre > 0)
   {
    if (!UpdateH || (M_ROW(UpdateH) != n_pre) || (M_COL(UpdateH) != n_pre))
     return Set_SVAR_Error(SVAR_SIZE_ERR);
    if (!Symmetric(UpdateH))
     return Set_SVAR_Error(SVAR_SYMMETRIC_ERR);
    if (!UpdateP || (M_ROW(UpdateP) != n_pre) || (M_COL(UpdateP) != n_var))
     return Set_SVAR_Error(SVAR_SIZE_ERR);
   }
  else
   if (UpdateH || UpdateP)
    return Set_SVAR_Error(SVAR_SIZE_ERR);
 return SVAR_NO_ERR;
}

/*
   Returns 0 if updates are OK and error code otherwise.  Sets SVAR_ERROR.
*/
int CheckUpdateArrays(TMatrix *UpdateS, TMatrix *UpdateH, TMatrix *UpdateP, int n_var, int n_pre)
{
 int i;
 if (!UpdateS || (MA_LEN(UpdateS) != n_var)) 
  return Set_SVAR_Error(SVAR_SIZE_ERR);
 if (n_pre > 0)
   {
    if (!UpdateH || (MA_LEN(UpdateH) != n_var) || !UpdateP || (MA_LEN(UpdateP) != n_var))
     return Set_SVAR_Error(SVAR_SIZE_ERR);
    for (i=0; i < n_var; i++) 
     if (CheckUpdateMatrices(UpdateS[i],UpdateH[i],UpdateP[i],n_var,n_pre)) 
      return Get_SVAR_Error();
   }
  else
   {
    if (UpdateH || UpdateP)
     return Set_SVAR_Error(SVAR_SIZE_ERR);
    for (i=0; i < n_var; i++) 
     if (CheckUpdateMatrices(UpdateS[i],(TMatrix)NULL,(TMatrix)NULL,n_var,n_pre))
      return Get_SVAR_Error();
   }
 return SVAR_NO_ERR;
}

/*
   Returns 0 if simulations are OK and error code otherwise.  Sets SVAR_ERROR.
*/
int CheckSimulations(T_SVAR *svar)
{
 int i;
 if (svar->n_pre > 0)
   for (i=svar->n_var-1; i >= 0; i--)
    {
     if (!svar->SimulateS[i] || !svar->SimulateH[i] || !svar->SimulateP[i])
      return Set_SVAR_Error(SVAR_SIMULATE_ERR);
    }
  else
   for (i=svar->n_var-1; i >= 0; i--)
    {
     if (!svar->SimulateS[i])
      return Set_SVAR_Error(SVAR_SIMULATE_ERR);
    } 
 return SVAR_NO_ERR;
}

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/


void PrintSVAR(FILE *f, T_SVAR *svar)
{
 int i;
 char format[]="%lf ";

 fprintf(f,"n_var = %d\n",svar->n_var);
 fprintf(f,"n_lag = %d\n",svar->n_lag);
 fprintf(f,"n_exg = %d\n",svar->n_exg);
 fprintf(f,"n_pre = %d\n",svar->n_pre);
 fprintf(f,"n_obs = %d\n",svar->n_obs);

 if (svar->U)
  for (i=0; i < svar->n_var; i++)
   {
    fprintf(f,"U[%d] = \n",i);
    PrintMatrix(f,svar->U[i],format);
   }
 if (svar->V)
  for (i=0; i < svar->n_var; i++)
   {
    fprintf(f,"V[%d] = \n",i);
    PrintMatrix(f,svar->V[i],format);
   }
 if (svar->W)
  for (i=0; i < svar->n_var; i++)
   {
    fprintf(f,"W[%d] = \n",i);
    PrintMatrix(f,svar->W[i],format);
   }

 if (svar->UpdateS)
  for (i=0; i < svar->n_var; i++)
   {
    fprintf(f,"UpdateS[%d] = \n",i);
    PrintMatrix(f,svar->UpdateS[i],format);
   }
 if (svar->UpdateH)
  for (i=0; i < svar->n_var; i++)
   {
    fprintf(f,"UpdateH[%d] = \n",i);
    PrintMatrix(f,svar->UpdateH[i],format);
   }
 if (svar->UpdateP)
  for (i=0; i < svar->n_var; i++)
   {
    fprintf(f,"UpdateP[%d] = \n",i);
    PrintMatrix(f,svar->UpdateP[i],format);
   }
  
 if (svar->SimulateS)
  for (i=0; i < svar->n_var; i++)
   {
    fprintf(f,"SimulateS[%d] = \n",i);
    PrintMatrix(f,svar->SimulateS[i],format);
   }
 if (svar->SimulateH)
  for (i=0; i < svar->n_var; i++)
   {
    fprintf(f,"SimulateH[%d] = \n",i);
    PrintMatrix(f,svar->SimulateH[i],format);
   }
 if (svar->SimulateP)
  for (i=0; i < svar->n_var; i++)
   {
    fprintf(f,"SimulateP[%d] = \n",i);
    PrintMatrix(f,svar->SimulateP[i],format);
   }
}


/******************************************************************************/
/************************ Maximum Likelihood Estimates ************************/
/******************************************************************************/
/*
    Assumes
      A_hat : (n_var x n_var) matrix

    Results
      Get the Maximum Likelihood estimate of the parameter A.
*/
TMatrix GetA_hat(TMatrix A_hat, T_SVAR *svar)
{
 PRECISION tol_chg=1E-6, tol_det=1.0;
 int max_iter=10000, ch;
 if (!A_hat)
   A_hat=CreateMatrix(svar->n_var,svar->n_var);
  else
   if ((svar->n_var != M_ROW(A_hat)) || (svar->n_var != M_COL(A_hat)))
    {
     MatrixError(SIZE_ERR);
     return (TMatrix)NULL;
    }
 if (!Maximize(A_hat,max_iter,tol_chg,tol_det,RANDOM_STARTING_VALUE,svar))
  do
   {
    printf("\nDesired Precision not reached in %d iterations.",max_iter);
    printf("\nMaximize further (Y)es/(N)o/(A)bort)? Y\n");
    ch=toupper(getchar());
    if (ch == 'A') exit(0);
    if (ch == 'N') break;
   }
  while (!Maximize(A_hat,max_iter,tol_chg,tol_det,CURRENT_STARTING_VALUE,svar));
 return(A_hat);
}

/*
    Assumes
      F_hat : (n_pre x n_var) matrix or null pointer.
      svar  : pointer to proper SVAR structure.
      A_hat : maximum likelihood estimate of A or null pointer. 

    Results
      Get the maximum likelihood estimate of F.  Allocates memory
      if F_hat is null.

    Returns
      F_hat upon success and null upon failure.  Null return 
      indicates either unable to allocate memory for F_hat or
      inability to get F_hat.

*/
TMatrix GetF_hat(TMatrix F_hat, T_SVAR *svar, TMatrix A_hat)
{
 int del=0, i, j, k;
 if (svar->n_pre > 0)
   {
    if (!F_hat)
      if (!(F_hat=CreateMatrix(svar->n_pre,svar->n_var)))
        {
         Set_SVAR_Error(SVAR_MEMORY_ERR);
         return (TMatrix)NULL;
        }
       else
        del=1;
     else
      if ((M_ROW(F_hat) != svar->n_pre) || (M_COL(F_hat) != svar->n_var))
       {
        Set_SVAR_Error(SVAR_SIZE_ERR);
        return (TMatrix)NULL;
       }
    if (!A_hat)
      if (!(A_hat=GetA_hat((TMatrix)NULL,svar)))
        {
         if (del) FreeMatrix(F_hat);
         return (TMatrix)NULL;
        }
       else
        del=1;
     else
      del=0;
    for (j=svar->n_var-1; j >= 0; j--)
     for (i=svar->n_pre-1; i >= 0; i--)
      for (F_hat[i][j]=0.0, k=svar->n_var-1; k >= 0; k--) 
       F_hat[i][j]+=svar->SimulateP[j][i][k]*A_hat[k][j];
    if (del) FreeMatrix(A_hat); 
    return F_hat;
   }
  else
   return (TMatrix)NULL;
}

/*
    Assumes
      B_hat : (n_pre x n_var) matrix or null pointer.
      svar  : Proper SVAR structure.
      A_hat : maximum likelihood estimate of A or null pointer. 

    Results
      Get the maximum likelihood estimate of the reduced form 
      parameter B = F*Inverse(A).  Allocates memory if B_hat 
      is null.

    Returns
      B_hat upon success and null upon failure.  Null return 
      indicates either unable to allocate memory for B_hat or
      inability to get A_hat.

*/
TMatrix GetB_hat(TMatrix B_hat, T_SVAR *svar, TMatrix A_hat)
{
 int del=0;
 if (svar->n_pre > 0)
   {
    if (!B_hat)
      if (!(B_hat=CreateMatrix(svar->n_pre,svar->n_var)))
        {
         Set_SVAR_Error(SVAR_MEMORY_ERR);
         return (TMatrix)NULL;
        }
       else
        del=1;
     else
      if ((M_ROW(B_hat) != svar->n_pre) || (M_COL(B_hat) != svar->n_var))
       {
        Set_SVAR_Error(SVAR_SIZE_ERR);
        return (TMatrix)NULL;
       }
    if (!A_hat)
     if (!(A_hat=GetA_hat((TMatrix)NULL,svar)))
       {
        if (del) FreeMatrix(B_hat);
        return (TMatrix)NULL;
       }
      else
       del=1;
    GetF_hat(B_hat,svar,A_hat);
    ProductInverseMM(B_hat,B_hat,A_hat);
    if (del) FreeMatrix(A_hat); 
    return B_hat;
   }
  else
   return (TMatrix)NULL;
}

/*
    Assumes
      Sigma_hat : (n_var x n_var) matrix
      A_hat     : (n_var x n_var) matrix of NULL pointer

    Results
      Get the maximum likelihood estimate of the variance-covariance
      matrix sigma.
*/
TMatrix GetSigma_hat(TMatrix Sigma_hat, T_SVAR *svar, TMatrix A_hat)
{
 int del=0;
 if (!Sigma_hat)
   {
    Sigma_hat=CreateMatrix(svar->n_var,svar->n_var);
    del=1;
   }
  else
   if ((M_ROW(Sigma_hat) != svar->n_var) || (M_COL(Sigma_hat) != svar->n_var))
    {
     Set_SVAR_Error(SVAR_MEMORY_ERR);
     return (TMatrix)NULL;
    }
 if (A_hat)
   ProductTransposeMM(Sigma_hat,A_hat,A_hat);
  else
   if (A_hat=GetA_hat((TMatrix)NULL,svar))
     {
      ProductTransposeMM(Sigma_hat,A_hat,A_hat);
      FreeMatrix(A_hat);
     }
    else
     {
      if (del) FreeMatrix(Sigma_hat);
      return (TMatrix)NULL;
     }
 if (!Inverse_Cholesky(Sigma_hat,Sigma_hat))
  {
   Set_SVAR_Error(SVAR_POSDEF_ERR);
   if (del) FreeMatrix(Sigma_hat);
   return (TMatrix)NULL;
  }

 return Sigma_hat;
}


/*
    Assumes
      A : (n_var x n_var) matrix
      F : (n_pre x n_var) matrix

    Results
      Returns natural logarithm of 
          
       (|det(A)|^n_obs)*exp(-0.5*a'*S_hat*a)
                  *exp(-0.5*(f - P_hat*a)'*H_hat*(f - P_hat*a))

      where 
        S_hat = diag(S_hat[0],...,S_hat[n_var-1])
        H_hat = diag(H_hat[0],...,H_hat[n_var-1])
        P_hat = diag(P_hat[0],...,P_hat[n_var-1])
        a'    = [a'[0],...,a'[n_var-1]]
        f'    = [f'[0],...,f'[n_var-1]]
        a[i]  = ith column of A
        f[i]  = ith column of F
       
    Notes 
      Uses the fact that the above is equal to

       n_obs*ln|det(A)| 
          - 0.5*(a'*UpdateS*a - 2*f'*UpdateP*a + f'*UpdateH*f)
          
      where 
        UpdateS = diag(UpdateS[0],...,UpdateS[n_var-1])
        UpdateH = diag(UpdateH[0],...,UpdateH[n_var-1])
        UpdateP = diag(UpdateP[0],...,UpdateP[n_var-1])
*/
PRECISION EvaluateLogLikelihood(TMatrix A, TMatrix F, T_SVAR *svar)
{
 int i, j, k;
 PRECISION x=0.0, y;
 for (k=svar->n_var-1; k >= 0; k--)
  {
   y=0;
   for (i=svar->n_var-1; i >= 0; i--)
    for (j=svar->n_var-1; j >= 0; j--)
     y+=A[i][k]*svar->UpdateS[k][i][j]*A[j][k];
   x+=y;
  }
 if (svar->n_pre > 0)
  for (k=svar->n_var-1; k >= 0; k--)
   {
    y=0;
    for (i=svar->n_pre-1; i >= 0; i--)
     for (j=svar->n_pre-1; j >= 0; j--)
      y+=F[i][k]*svar->UpdateH[k][i][j]*F[j][k];
    x+=y;
    y=0;
    for (i=svar->n_pre-1; i >= 0; i--)
     for (j=svar->n_var-1; j >= 0; j--)
      y+=F[i][k]*svar->UpdateP[k][i][j]*A[j][k];
    x-=2.0*y;
   }
 y=fabs(Determinant_LU(A));
 return (y > 0.0) ? (PRECISION)(svar->n_obs)*log(y) - 0.5*x : -1.0E30;
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/******************************** Maximization ********************************/
/*
    Assumes
      A_hat    : (n_var x n_var) matrix
      max_iter : maximum number of interations allowed
      tol      : iterates until the percentage change is less than tol
      flag     : controls starting value

    Results
      A_hat is the matrix which maximizes

        (|det([a[1] ... a[n]])|^n_obs)*exp(-0.5*a'*U*inverse(S)*U'*a)

      subject to the constraint Q[i]*a[i]=0.

      Returns the 1 on success, 0 otherwise

    Notes
      tol_chg is the percentage change of A_hat under the usual Euclidean norm.
      tol_det is the percentage change in the determinant of A_hat.
*/
int Maximize(TMatrix A_hat, int max_iter, PRECISION tol_chg, PRECISION tol_det, int flag, T_SVAR *svar)
{
 int i, j, k, count=0;
 TVector u, w;
 PRECISION norm, chg, tmp, sqrt_obs=sqrt(svar->n_obs), new_det, old_det;

 switch (flag)
  {
   case RANDOM_STARTING_VALUE: UniformSphericalInitialValue(A_hat,svar); break;
   case CURRENT_STARTING_VALUE: break;
   default:
    printf("\nMaximize:  flag not recognized\n");
    exit(0);
  }

 w=CreateVector(svar->n_var);

 tol_chg*=tol_chg/10000.0;
 tol_det=1.0+tol_det/100.0;

 new_det=fabs(Determinant_LU(A_hat));

 do
  {
   norm=chg=0.0;
   old_det=new_det;

   for (k=svar->n_var-1; k >= 0; k--)
    {
     /* Get non-zero vector w perpendicular to all columns of A_hat except */
     /* the kth column.                                                    */
     GetNullVector(w,A_hat,k);

     /* u = SimulateS'[k] * w */
     u=ProductVM((TVector)NULL,w,svar->SimulateS[k]);

     /* Normalize u */
     ProductSV(u,sqrt_obs/Norm(u),u);

     /* w = SimulateS[k] * u */
     ProductMV(w,svar->SimulateS[k],u);

     FreeVector(u);

     /* Normalize w so that the largest element of w is of the same sign as */
     /* the corresponding element in A_hat.  Compute the norm of A_hat and  */
     /* the change in A_hat                                                 */
     for (tmp=fabs(w[0]), i=0, j=svar->n_var-1; j > 0; j--)
      if (fabs(w[j]) > tmp)
       {
        tmp=fabs(w[j]);
        i=j;
       }
     if (w[i]*A_hat[i][k] < 0)
       for (i=svar->n_var-1; i >= 0; i--)
        {
         tmp=A_hat[i][k];
         norm+=tmp*tmp;
         tmp+=w[i];
         chg+=tmp*tmp;
         A_hat[i][k]=-w[i];
        }
      else
       for (i=svar->n_var-1; i >= 0; i--)
        {
         tmp=A_hat[i][k];
         norm+=tmp*tmp;
         tmp-=w[i];
         chg+=tmp*tmp;
         A_hat[i][k]=w[i];
        }
    }

   new_det=fabs(Determinant_LU(A_hat));
   if ((new_det > tol_det*old_det) && (chg <= tol_chg*norm))  {printf("!! %lf %lf\n",new_det,old_det); getchar();}
  }
 while (((new_det > tol_det*old_det) || (chg > tol_chg*norm)) && (++count < max_iter));



 FreeVector(w);

 return (count >= max_iter) ? 0 : 1;
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/******************************************************************************/
/********************************* Simulation *********************************/
/******************************************************************************/
/*
    Assumes
      A : (n_var x n_var) matrix

    Results
      Randomly selects A subject to the constraints.  The matrix A is not 
      normalized.
*/
void UniformSphericalInitialValue(TMatrix A, T_SVAR *svar)
{
 int i, j;
 TVector u, v;

 v=CreateVector(svar->n_var);

 for (j=svar->n_var-1; j >= 0; j--)
  {
   u=NormalVector(CreateVector(M_COL(svar->U[j])));
   ProductSV(u,1.0/Norm(u),u);
   ProductMV(v,svar->SimulateS[j],u);
   FreeVector(u);
   for (i=svar->n_var-1; i >= 0; i--) A[i][j]=v[i];
  }

 FreeVector(v);
}

/*
    Assumes
      A : (n_var x n_var) matrix

    Results
      Randomly selects A subject to the constraints. UniformSphericalInitialValue() 
      is called.  Then n calls of Simulate_A() are made.
*/
void RandomInitialValue(TMatrix A, int n, T_SVAR *svar)
{
 UniformSphericalInitialValue(A,svar);
 while (n-- > 0) Simulate_A(A,svar);
}

/*
    Assumes
      A : (n_var x n_var) matrix

    Results
      Randomly selects A subject to the constraints Q[i].  A is set to A_hat and
      then n calls of Simulate_A() are made.
*/
void MLEInitialValue(TMatrix A, int n, T_SVAR *svar)
{
 GetA_hat(A,svar);
 while (n-- > 0) Simulate_A(A,svar);
}


/*
    Assumes
      A     : (n_var x n_var) matrix

    Results
      Draws b from the distribution whose density function is
      proportional to

       (|det([U[1]*b[1] ... U[n]*b[n]])|^n_obs)*exp(-0.5*b'(inverse(S))b)

      a[i] is set to U[i]*b[i]

    Notes
      Uses a Gibbs sampler to draw A conditional passed value of A.
*/
void Simulate_A(TMatrix A, T_SVAR *svar)
{
 int i, j, k, q, m;
 TVector u, v, w;
 PRECISION c0, c1, scale, sqrt_nobs=sqrt(svar->n_obs);

 w=CreateVector(svar->n_var);

 for (k=svar->n_var-1; k >= 0; k--)
  {
   /* Get non-zero vector w perpendicular to all columns of A except the kth */
   GetNullVector(w,A,k);

   /* u = SimulateS'[k] * w */
   u=ProductVM((TVector)NULL,w,svar->SimulateS[k]);

   /* Normalize u and draw univariate Wishard component */
   v=ProductSV((TVector)NULL,sqrt_nobs*Simulate(svar->n_obs,0)/Norm(u),u);

   /* Find largest element of u */
   for (scale=fabs(u[0]), m=0, i=(q=M_COL(svar->U[k]))-1; i > 0; i--)
    if (fabs(u[i]) > scale)
     {
      scale=fabs(u[i]);
      m=i;
     }
   c0=scale*scale;

   /* Draw the Gaussian components */
   for (j=0; j < m; j++)
    {
     c1=c0+u[j]*u[j];
     scale=gaussian()*sqrt(c0/c1);
     v[j]-=scale;
     scale*=u[j]/c0;
     for (i=0; i < j; i++) v[i]+=scale*u[i];
     v[m]+=scale*u[m];
     c0=c1;
    }
   for (j=m+1; j < q; j++)
    {
     c1=c0+u[j]*u[j];
     scale=gaussian()*sqrt(c0/c1);
     v[j]-=scale;
     scale*=u[j]/c0;
     for (i=0; i < j; i++) v[i]+=scale*u[i];
     c0=c1;
    }

   /* w = Usqrts[k] * v */
   ProductMV(w,svar->SimulateS[k],v);

   for (i=svar->n_var-1; i >= 0; i--) A[i][k]=w[i];

   FreeVector(u);
   FreeVector(v);
  }

 FreeVector(w);
}

/*
    Assumes
      A    : (n_var x n_var) matrix
      F    : (n_pre x n_var) matrix or NULL pointer if n_pre = 0
      svar : valid pointer to T_SVAR type

    Results
      Conditional on A, g[i] is drawn from normal density with mean P[i]*b[i]
      and variance H[i].  f[i] is set to V[i]*g[i] - W[i]*U[i]*b[i].
*/
void Simulate_F(TMatrix A, TMatrix F, T_SVAR *svar)
{
 int i, j, k;
 TVector x, y;
 PRECISION tmp;

 if (svar->n_pre > 0)
  {
   x=CreateVector(svar->n_pre);
   for (j=svar->n_var-1; j >= 0; j--)
    {
     if (svar->SimulateH[j])
       {
        ProductMV(x,svar->SimulateH[j],y=NormalVector(CreateVector(M_COL(svar->SimulateH[j]))));
        FreeVector(y);
        for (i=svar->n_pre-1; i >= 0; i--)
         {
          tmp=x[i];
          for (k=svar->n_var-1; k >= 0; k--) tmp+=svar->SimulateP[j][i][k]*A[k][j];
          F[i][j]=tmp;
         }
       }
      else    
       for (i=svar->n_pre-1; i >= 0; i--)
        {
         tmp=0.0;
         for (k=svar->n_var-1; k >= 0; k--) tmp+=svar->SimulateP[j][i][k]*A[k][j];
         F[i][j]=tmp;
        }
    }
   FreeVector(x);
  }
}

/*
    Assumes
      A : (n_var x n_var) matrix
      F : (n_pre x n_var) matrix or NULL pointer if n_pre = 0

    Results
      Draws b from the distribution whose density function is
      proportional to

       (|det([U[1]*b[1] ... U[n]*b[n]])|^n_obs)*exp(-0.5*b'(inverse(S))b)

      a[i] is set to U[i]*b[i]

      Conditional on b[i], g[i] is drawn from normal density with mean
      P[i]*b[i] and variance H[i].  f[i] is set to V[i]*g[i] - W[i]*U[i]*b[i]

    Notes
      Uses a Gibbs sampler to draw A conditional passed value of A.
*/
void Simulate_Full(TMatrix A, TMatrix F, T_SVAR *svar)
{
 Simulate_A(A,svar);
 Simulate_F(A,F,svar);
}

/*
    Assumes
      A : (n_var x n_var) matrix
      B : (n_pre x n_var) matrix of NULL pointer if n_pre = 0.

    Results
      Draws b from the distribution whose density function is
      proportional to

       (|det([U[1]*b[1] ... U[n]*b[n]])|^n_obs)*exp(-0.5*b'(inverse(S))b)

      a[i] is set to U[i]*b[i]

      Conditional on b[i], g[i] is drawn from normal density with mean
      P[i]*b[i] and variance H[i].  f[i] is set to V[i]*b[i].  

      B is set to F*Inverse(A).

    Notes
      Uses a Gibbs sampler to draw A conditional passed value of A.
*/
void Simulate_ReducedForm(TMatrix A, TMatrix B, T_SVAR *svar)
{
 Simulate_Full(A,B,svar);

 if ((svar->n_pre > 0) && !ProductInverseMM(B,B,A))
  {
   printf("\nSimulate_ReducedForm:  A not of full rank\n");
   exit(0);
  }
}

/*
    Assumes
      A     : (n_var x n_var) matrix
      Sigma : (n_var x n_var) matrix

    Results
      Draws b from the distribution whose density function is
      proportional to

       (|det([U[1]*b[1] ... U[n]*b[n]])|^n_obs)*exp(-0.5*b'(inverse(S))b)

      a[i] is set to U[i]*b[i]

      Sigma is set to Inverse(A*A').

    Notes
      Uses a Gibbs sampler to draw A conditional passed value of A.

      The returned value of A has been normalized according to the contents
      of NormalizationFlag.
*/
void Simulate_Sigma(TMatrix A, TMatrix Sigma, T_SVAR *svar)
{
 Simulate_A(A,svar);

 ProductTransposeMM(Sigma,A,A);

 if (!Inverse_LU(Sigma,Sigma))
  {
   printf("\nSimulate_Sigma:  A not of full rank\n");
   exit(0);
  }
}

/*
    Assumes
      A     : (n_var x n_var) matrix
      Sigma : (n_var x n_var) matrix
      B     : (n_pre x n_var) matrix or NULL pointer if n_pre = 0.

    Results
      Draws b from the distribution whose density function is
      proportional to

       (|det([U[1]*b[1] ... U[n]*b[n]])|^n_obs)*exp(-0.5*b'(inverse(S))b)

      a[i] is set to U[i]*b[i]

      Conditional on b[i], g[i] is drawn from normal density with mean
      P[i]*b[i] and variance H[i].  f[i] is set to V[i]*b[i].  

      B is set to F*Inverse(A).  Sigma is set to Inverse(A*A').

    Notes
      Uses a Gibbs sampler to draw A conditional passed value of A.
*/
void Simulate_FullReducedForm(TMatrix A, TMatrix Sigma, TMatrix B, T_SVAR *svar)
{
 Simulate_ReducedForm(A,B,svar);

 ProductTransposeMM(Sigma,A,A);

 if (!Inverse_LU(Sigma,Sigma))
  {
   printf("\nSimulate_ReducedFormFull:  A not of full rank\n");
   exit(0);
  }
}

/******************************************************************************/


/******************************************************************************/
/********************************* Utilities **********************************/
/******************************************************************************/
#ifndef USE_SINGULAR_VALUE_DECOMPOSITION
/*
   Assumes
    v : n-vector
    X : n x n matrix
    j : column of X implicitly set to 0

   Results
    Solves v*X=0 for non-zero v.

   Notes
    Uses Gaussian elimination and back substitution to solve u*X = 0.
*/
void GetNullVector(TVector v, TMatrix X, int j)
{
 int i, k, jmax, n=M_ROW(X);
 double big, sum, tmp;
 TVector scale;
 TMatrix Y;

 if ((n != M_COL(X)) || (n != V_DIM(v)))
  MatrixError(SIZE_ERR);

 Y=EquateMatrix(CreateMatrix(n,n),X);

 /* Set jth column equal to the last column */
 if (j < n-1)
  for (i=n-1; i >= 0; i--) Y[i][j]=Y[i][n-1];

 scale=CreateVector(n-1);

 /* Implicit scaling */
 for (j=n-2; j >= 0; j--)
  {
   big=fabs(Y[0][j]);
   for (i=n-1; i > 0; i--) if ((tmp=fabs(Y[i][j])) > big) big=tmp;
   scale[j]=(big == 0.0) ? 0 : 1.0/big;
  }

 for (i=0; i < n-1; i++)
  {
   /* Find pivot */
   big=scale[i]*fabs(Y[i][i]);
   jmax=i;
   for (j=i+1; j < n-1; j++)
    if ((tmp=scale[j]*fabs(Y[i][j])) > big)
     {
      big=tmp;
      jmax=j;
     }
                      
   /* Is pivot zero? */
   if (big == 0) break;

   /* Interchange columns if necessary */
   if (jmax != i)
    {
     for (k=i; k < n; k++)
      {
       tmp=Y[k][i];
       Y[k][i]=Y[k][jmax];
       Y[k][jmax]=tmp;
      }
     scale[jmax]=scale[i];
    }
 
   /* Zero out the ith row */
   big=1.0/Y[i][i];
   for (j=i+1; j < n-1; j++)
    for (tmp=-Y[i][j]*big, k=i+1; k < n; k++)
     Y[k][j]+=tmp*Y[k][i];
  }

 /* Back substitution */
 for (k=n-1; k > i; k--) v[k]=0;
 v[i]=tmp=1.0;
 for (k=i-1; k >= 0; k--)
  {
   for (sum=-Y[i][k], j=i-1; j > k; j--) sum-=v[j]*Y[j][k];
   v[k]=sum/Y[k][k];
   tmp+=v[k]*v[k];
  }
 tmp=1.0/sqrt(tmp);
 for (k=i; k >= 0; k--) v[k]*=tmp;

 FreeVector(scale);
 FreeMatrix(Y);
}
#endif

/*
   Assumes
    v : n-vector
    X : n x n matrix
    j : column of X implicitly set to 0

   Results
    Solves v*X = 0 for non-zero v.

   Notes
    Uses singular value decomposition to solve for u*X = 0.
*/
#ifdef USE_SINGULAR_VALUE_DECOMPOSITION
void GetNullVector(TVector v, TMatrix X, int j)
{
 int i, k, n=M_ROW(X);
 PRECISION tmp;
 TMatrix Y, U, V;

#ifdef CHECK_SIZE
 if ((n != M_COL(X)) || (n != V_DIM(v)) || (j < 0) || (j >= n))
  MatrixError(SIZE_ERR);
#endif

 Y=EquateMatrix(CreateMatrix(n,n),X);
 U=CreateMatrix(n,n);
 V=CreateMatrix(n,n);

 /* Set jth column equal zero */
 for (i=n-1; i >= 0; i--) Y[i][j]=0.0;

 if (!SingularValueDecomposition(U,v,V,Y))
  {
   printf("\nGetNullVector:  Unable to obtain Singular Value Decomposition\n");
   exit(0);
  }

 /* Find smallest singular value */
 for (tmp=fabs(v[k=0]), i=n-1; i > 0; i--) if (v[i] < tmp) tmp=fabs(v[k=i]);

 for (i=n-1; i >=0; i--) v[i]=U[i][k];

 FreeMatrix(Y);
 FreeMatrix(U);
 FreeMatrix(V);
}
#endif

/*
   Assumes
    X - (? x n) matrix or null pointer

   Returns
    Upon success, returns a (n x q) matrix whose columns are
    an orthonormal basis for the null space of X, or a null
    pointer if the null space is 0-dimensional.  Upon failure,
    returns null pointer and calls Set_SVAR_Error().  Does not
    reset SVAR_ERR upon entry.

   Results
    Creates and returns orthonormal basis for null space of X.
    If X is null pointer, then returns a (n x n) identity matrix.
   
   Returns
*/
TMatrix GetNullSpace(TMatrix X, int n)
{
 TMatrix U, V, B=(TMatrix)NULL;
 TVector D;
 PRECISION max;
 int i, j, q;

 if (!X) 
   {
    if (!(B=IdentityMatrix((TMatrix)NULL,n)))
     Set_SVAR_Error(SVAR_MEMORY_ERR);
   }
  else
   if (M_COL(X) != n) 
     Set_SVAR_Error(SVAR_SIZE_ERR);
    else
     {
      if (SVD(U=CreateMatrix(M_ROW(X),n),D=CreateVector(n),V=CreateMatrix(n,n),X))
        {
         for (max=D[0], j=n-1; j > 0; j--) 
          if (max < fabs(D[j])) max=fabs(D[j]);
         max*=SQRT_MACHINE_EPSILON;
         for (q=0, j=n-1; j >= 0; j--)
          if (fabs(D[j]) <= max) q++;
         if (q > 0)
          if (!(B=CreateMatrix(n,q)))
            Set_SVAR_Error(SVAR_MEMORY_ERR);
           else
            for (j=n-1; j >= 0; j--)
             if (fabs(D[j]) <= max)
              for (q--, i=n-1; i >= 0; i--)
               B[i][q]=V[i][j];
        }
       else
        Set_SVAR_Error(SVAR_DECOMPOSITION_ERR);
      FreeMatrix(U);
      FreeMatrix(V);
      FreeVector(D);
     }
 return B;
}

/*
   Assumes
     X : k x q matrix
     Y : k x k symmetric positive definite matrix

   Results
     Allocates memory for and computes X'*Y*X.

   Returns
     X'*Y*X upon success and null otherwise. 
*/
TMatrix SymmetricForm(TMatrix X, TMatrix Y)
{
 TMatrix A, B;
 if (!X || !Y || (M_ROW(X) != M_COL(Y)) || (M_ROW(X) != M_ROW(Y))) 
  return (TMatrix)NULL; 
 B=TransposeProductMM((TMatrix)NULL,X,A=ProductMM((TMatrix)NULL,Y,X));
 FreeMatrix(A);
 return B;
}

/*
   Assumes
     X : k x q matrix
     Y : k x k symmetric positive definite matrix

   Results
     Allocates memory for and computes X*Sqrt(Inverse((X'*Y*X)) where
     Sqrt(Z)*Sqrt(Z)'=Z.

   Returns
      X*Sqrt(Inverse((X'*Y*X)) upon success and null otherwise. 

   Notes
     Failure usually indicates X'*Y*X is not positive definite.
*/
TMatrix MakeSimulationMatrix(TMatrix X, TMatrix Y)
{
 TMatrix A, B; 
 if (!(B=SymmetricForm(X,Y))) return (TMatrix)NULL;
 A=(CholeskyLT(B,B) && Inverse_LT(B,B)) ? ProductMM((TMatrix)NULL,X,B) : (TMatrix)NULL;
 FreeMatrix(B);
 return A;
}

/*
   Returns 1 if X is symmetric, 0 otherwise.
*/
int Symmetric(TMatrix X)
{
 int i, j;
 if (M_ROW(X) != M_COL(X)) return 0;
 for (i=M_ROW(X)-1; i >= 0; i--)
  for (j=M_COL(X)-1; j > i; j--)
   if (fabs(X[i][j]-X[j][i]) > MACHINE_EPSILON*((fabs(X[i][j]) > fabs(X[j][i])) ? fabs(X[i][j]) : fabs(X[j][i]))) 
    return 0;
 return 1;
}

/*
   Returns 1 if the columns of X are orthonormal, 0 otherwise.
*/
int OrthonormalColumns(TMatrix X)
{
 int i, j, n=M_COL(X), result=1;
 TMatrix Y=TransposeProductMM((TMatrix)NULL,X,X);
 for (i=n-1; i >= 0; i--)
  {
   for (j=n-1; j > i; j--) if (fabs(Y[i][j]) > SQRT_MACHINE_EPSILON) result=0;
   if (fabs(1.0-Y[i][i]) > SQRT_MACHINE_EPSILON) result=0;
  }
 FreeMatrix(Y);
 return result;
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

/******************************************************************************/
/******************************** Normalization *******************************/
/******************************************************************************/
/*
   Assumes
    A    : n x n matrix
    Ref  : n x n matrix

   Results
    A is normalized so that the diagonal of Inverse(A)*Ref is positive.

   Notes
    The normalization described above is equivalent to requiring that a[k]
    and the kth column of Ref lie on the same side of the hyperplane spanned
    by a[0],...,a[k-1],a[k+1],...,a[k]

*/
void Normalize_WZ(TMatrix A, TMatrix Ref)
{
 int i, j, n=M_COL(A);
 TMatrix B=CreateMatrix(n,n);

 if (!InverseProductMM(B,A,Ref))
  {
   printf("\nNormalizeWZ:  A not invertible\n");
   exit(0);
  }

 for (j=n-1; j >= 0; j--)
  if (B[j][j] < 0) for (i=n-1; i >= 0; i--) A[i][j]=-A[i][j];

 FreeMatrix(B);
}

void Simulate_WZ(TMatrix A, TMatrix Ref, T_SVAR *svar)
{
  Simulate_A(A,svar);
  Normalize_WZ(A,Ref);
}

void Simulate_Full_WZ(TMatrix A, TMatrix F, TMatrix Ref, T_SVAR *svar)
{
  Simulate_A(A,svar);
  Normalize_WZ(A,Ref);
  Simulate_F(A,F,svar);
}

/*
   Assumes
    A    : n x n matrix
    Ref  : n x n matrix

   Results
    A is normalized so that the diagonal of A'*Ref is positive.

   Notes
    If Ref has exactly one non-zero element in each column, this normalization
    is equivalent to requiring that the corresponding elements of A are
    positive.
*/
void Normalize_Traditional(TMatrix A, TMatrix Ref)
{
 int i, j, n=M_COL(A);
 PRECISION x;

 for (j=n-1; j >= 0; j--)
  {
   for (x=A[0][j]*Ref[0][j], i=n-1; i > 0; i--) x+=A[i][j]*Ref[i][j];
   if (x < 0) for (i=n-1; i >= 0; i--) A[i][j]=-A[i][j];
  }
}

void Simulate_A_Traditional(TMatrix A, TMatrix Ref, T_SVAR *svar)
{
  Simulate_A(A,svar);
  Normalize_Traditional(A,Ref);
}

void Simulate_Full_Traditional(TMatrix A, TMatrix F, TMatrix Ref, T_SVAR *svar)
{
  Simulate_A(A,svar);
  Normalize_Traditional(A,Ref);
  Simulate_F(A,F,svar);
}

/*
   Assumes
    A: n x n matrix

   Results
    A is normalized so that the diagonal is positive.
*/
void Normalize_Diagonal(TMatrix A)
{
 int i, j, n=M_COL(A);

 for (j=n-1; j >= 0; j--)
  if (A[j][j] < 0) for (i=n-1; i >= 0; i--) A[i][j]=-A[i][j];
}

void Simulate_A_Diagonal(TMatrix A, T_SVAR *svar)
{
  Simulate_A(A,svar);
  Normalize_Diagonal(A);
}

void Simulate_Full_Diagonal(TMatrix A, TMatrix F, T_SVAR *svar)
{
  Simulate_A(A,svar);
  Normalize_Diagonal(A);
  Simulate_F(A,F,svar);
}
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/


#ifdef USE_SINGULAR_VALUE_DECOMPOSITION
  #undef USE_SINGULAR_VALUE_DECOMPOSITION
#endif
#ifdef VERBOSE
  #undef VERBOSE
#endif


