
#include "wald.h"

/******************************************************************************/
/**************************** Information Matrices ****************************/
/******************************************************************************/
/*
   X : (m*n x m*n) matrix or null pointer

   Returns the (m,n) commutation matrix in X.  This matrix is allocated if X 
   is the null pointer.  This matrix has the property that if A is any m x n 
   matrix then

                                vec(A') = X*vec(A)
*/
TMatrix CommutationMatrix(TMatrix X, int m, int n)
{
 int i=m*n, j;
 if (X)
   {
    if ((M_ROW(X) != i) || (M_COL(X) != i))
     {
      MatrixError(SIZE_ERR);
      return (TMatrix)NULL;
     }
   }
  else
   {
    if (!(X=CreateMatrix(i,i))) return (TMatrix)NULL;
   }
 InitializeMatrix(X,0.0);
 for (j=n-1; j >= 0; j--)
  for (i=m-1; i >= 0; i--)
   X[i*n+j][j*m+i]=1.0;
 return X;
}

/*
   Data from a vector autoregression can be used to approximate
   the information matrix defined above.  The approximation is
   given by 

        H'*K(n,n)*(Inverse(A)' x Inverse(A))*H + (I(n) x S)/T

   where 
               ---
               | 1 if r = j*n+i and s = j*(n+k)+i for some 0 <= i,j < n     
       H(r,s) =|
               | 0 otherwise
               ---

       S = W' * W

           ---                                  ---
           |  y(1)   -y(0)  ...  -y(1-p)   -z(1)  |
           |  y(2)   -y(1)  ...  -y(2-p)   -z(2)  |
           |   .       .             .       .    |
       W = |   .       .             .       .    |
           |   .       .             .       .    |
           | y(T-1) -y(T-2) ... -y(T-1-p) -z(T-1) |
           |  y(T)  -y(T-1) ...  -y(T-p)   -z(T)  |
           ---                                  ---

   Note that 

            --      --
            | y(1-p) |
            | y(2-p) |
            |   .    |
     Data = |   .    |
            |   .    |
            | y(T-1) |
            |  y(T)  |
            --      --

   and

           --      --
           |  z(1)  |
           |  z(2)  |
           |   .    |
       W = |   .    |
           |   .    |
           | z(T-1) |
           |  z(T)  |
           --      -- 
*/
TMatrix InformationMatrix_2ndDerivative(TMatrix A, TMatrix F, TMatrix Data, TMatrix Exg, int p)
{
 TMatrix A_inverse, A_inverse_transpose, H, K, W, S, I, T1, T2, T3, T4, T5;
 int i, j, n=M_COL(Data), m=Exg ? M_COL(Exg) : 0, k=n*p+m, T=M_ROW(Data)-p, r;

 if ((M_ROW(A) != n) || (M_COL(A) != n) || (F && ((M_ROW(F) != k) || (M_COL(F) != n))) || (Exg && (M_ROW(Exg) != T)))
   {
     printf("Error in InformationMatrix_1stDerivative");
     exit(0);
   } 

 H=InitializeMatrix(CreateMatrix(n*n,n*(n+k)),0.0);
 for (i=n-1; i >= 0; i--)
   for (j=n-1; j >= 0; j--)
     H[j*n+i][j*(n+k)+i]=1.0;

 K=CommutationMatrix((TMatrix)NULL,n,n);

 A_inverse=Inverse_LU((TMatrix)NULL,A);
 A_inverse_transpose=Transpose((TMatrix)NULL,A_inverse);
 
 W=CreateMatrix(T,n+k);
 for (i=0; i < M_ROW(W); i++)
   {
     for (j=0; j < n; j++)
       {
	 W[i][j]=Data[i+p][j];
	 for (r=1; r <= p; r++)
	   W[i][r*n+j]=-Data[i+p-r][j];
       }
     for (j=0; j < m; j++)
       W[i][(p+1)*n+j]=-Exg[i][j];
   }
 S=TransposeProductMM((TMatrix)NULL,W,W);

 I=IdentityMatrix((TMatrix)NULL,n);

 T1=KroneckerProduct((TMatrix)NULL,A_inverse_transpose,A_inverse);
 T2=ProductMM((TMatrix)NULL,K,T1);
 T3=TransposeProductMM((TMatrix)NULL,H,T2);
 T4=ProductMM((TMatrix)NULL,T3,H);
 T5=KroneckerProduct((TMatrix)NULL,I,S);
 ProductMS(T5,T5,1.0/(PRECISION)T);
 AddMM(T5,T4,T5);

 FreeMatrix(T4);
 FreeMatrix(T3);
 FreeMatrix(T2);
 FreeMatrix(T1);
 FreeMatrix(I);
 FreeMatrix(S);
 FreeMatrix(W);
 FreeMatrix(A_inverse_transpose);
 FreeMatrix(A_inverse);
 FreeMatrix(K);
 FreeMatrix(H);

 return T5;
}

/*
   Data from a vector autoregression can be used to approximate
   the information matrix defined above.  The approximation is
   given by 

               T-1
               --- 
               \    
                |   v(t) * v(t)'
               /
               ---
               t=0

   where 
                                                     -   -
                                                     | A |
       v(t) = H'*vec(Inverse(A)') - (I(n) x S(t))*vec|   |
                                                     | F |
                                                     -   -

               ---
               | 1 if r = j*n+i and s = j*(n+k)+i for some 0 <= i,j < n     
       H(r,s) =|
               | 0 otherwise
               ---

       S = W(t)' * W(t)

              --                                  --
       W(t) = | y(t+1) -y(t) ... -y(t+1-p) -z(t+1) |
              --                                  --

   Note that

            --      --
            | y(1-p) |
            | y(2-p) |
            |   .    |
     Data = |   .    |
            |   .    |
            | y(T-1) |
            |  y(T)  |
            --      --

   and

           --      --
           |  z(1)  |
           |  z(2)  |
           |   .    |
       W = |   .    |
           |   .    |
           | z(T-1) |
           |  z(T)  |
           --      -- 
*/
TMatrix InformationMatrix_1stDerivative(TMatrix A, TMatrix F, TMatrix Data, TMatrix Exg, int p)
{
 TMatrix A_inverse, A_inverse_transpose, H, K, S, I, Rtrn;
 TVector a, b, c, v, w;
 int i, j, t, n=M_COL(Data), m=Exg ? M_COL(Exg) : 0, k=n*p+m, T=M_ROW(Data)-p, r;
 
 if ((M_ROW(A) != n) || (M_COL(A) != n) || (F && ((M_ROW(F) != k) || (M_COL(F) != n))) || (Exg && (M_ROW(Exg) != T)))
   {
     printf("Error in InformationMatrix_1stDerivative");
     exit(0);
   } 

 Rtrn=InitializeMatrix(CreateMatrix(n*(n+k),n*(n+k)),0.0);

 /**
 return IdentityMatrix(Rtrn,n*(n+k));
 /**/

 H=InitializeMatrix(CreateMatrix(n*n,n*(n+k)),0.0);
 for (i=n-1; i >= 0; i--)
   for (j=n-1; j >= 0; j--)
     H[j*n+i][j*(n+k)+i]=1.0;

 A_inverse=Inverse_LU((TMatrix)NULL,A);
 A_inverse_transpose=Transpose((TMatrix)NULL,A_inverse);
 c=Vec((TVector)NULL,A_inverse_transpose);
 a=TransposeProductMV((TVector)NULL,H,c);

 b=CreateVector(n*(n+k));
 for (j=0; j < n; j++)
   {
     for (i=0; i < n; i++)
       b[j*(n+k)+i]=A[i][j];
     for (i=0; i < k; i++)
       b[j*(n+k)+n+i]=F[i][j];
   }

 I=IdentityMatrix((TMatrix)NULL,n);
 w=CreateVector(n+k);
 S=CreateMatrix(n+k,n+k);
 K=CreateMatrix(n*(n+k),n*(n+k));
 v=CreateVector(n*(n+k));

 for (t=0; t < T; t++)
   {
     for (j=0; j < n; j++)
       {
	 w[j]=Data[t+p][j];
	 for (r=1; r <= p; r++)
	   w[r*n+j]=-Data[t+p-r][j];
       }
     for (j=0; j < m; j++)
       w[(p+1)*n+j]=-Exg[t][j];

     OuterProduct(S,w,w);
     KroneckerProduct(K,I,S);
     ProductMV(v,K,b);
     SubtractVV(v,a,v);
     for (i=n*(n+k)-1; i >= 0; i--)
       for (j=n*(n+k)-1; j>= 0; j--)
	 Rtrn[i][j]+=v[i]*v[j];
   }

 FreeVector(v);
 FreeMatrix(K);
 FreeMatrix(S);
 FreeVector(w);
 FreeMatrix(I);
 FreeVector(b);
 FreeVector(a);
 FreeVector(c);
 FreeMatrix(A_inverse_transpose);
 FreeMatrix(A_inverse);
 FreeMatrix(H);

 ProductMS(Rtrn,Rtrn,1.0/(PRECISION)T);
 return Rtrn;
}

/*
   Normalize using the information matrix at the point A_hat.

   A, F        - parameters to be normalized.
   P->A, P->F  - reference matrices.
   P->S        - information matrix at reference matrices.

   Chooses the sign of each column of A to minimize

                       v' * IM * v
   
   where
                          -   -
                          | A |
                  v = Vec |   |
                          | F |
                          -   -

*/
void NormalizeWald(TMatrix A, TMatrix F, TMatrix A_ref, TMatrix F_ref, TMatrix IM)
{
  TVector v;
  int *r, *s, *t, i, j, k, n=M_ROW(A), m=F ? M_ROW(F) : 0;
  PRECISION min, tmp;
  s=(int*)malloc(3*n*sizeof(int));
  t=s+n;
  r=t+n;
  for (j=n-1; j >= 0; j--) 
    {
      s[j]=t[j]=1;
      r[j]=0;
    }
  v=CreateVector(n*(m+n));
  for (k=n*(m+n), j=n-1; j >= 0; j--)
    {
      for (i=m-1; i >= 0; i--)
	v[--k]=F[i][j] - F_ref[i][j];
      for (i=n-1; i >= 0; i--)
	v[--k]=A[i][j] - A_ref[i][j];
    }
  min=InnerProduct(v,v,IM);
  j=0;
  do
    {
      r[j]=1;
      if ((t[j]=-t[j]) < 0)
	{
	  for (k=(j+1)*(n+m), i=m-1; i >= 0; i--)
	    v[--k]=-F[i][j] - F_ref[i][j];
	  for (i=n-1; i >= 0; i--)
	    v[--k]=-A[i][j] - A_ref[i][j];
	}
      else
	{
	  for (k=(j+1)*(n+m), i=m-1; i >= 0; i--)
	    v[--k]=F[i][j] - F_ref[i][j];
	  for (i=n-1; i >= 0; i--)
	    v[--k]=A[i][j] - A_ref[i][j];
	    }
      if ((tmp=InnerProduct(v,v,IM)) < min)
	{
	  memcpy(s,t,n*sizeof(int));
	  min=tmp;
	}
      for (j=0; (j < n) && r[j]; j++) r[j]=0;
    }
  while (j < n);
 
  for (j=n-1; j >= 0; j--)
    if (s[j] < 0)
      {
	for (i=n-1; i >= 0; i--) A[i][j]=-A[i][j];
	for (i=m-1; i >= 0; i--) F[i][j]=-F[i][j];
	}
         
      FreeVector(v);
  free(s);
}

/****** OLD STUFF ******/
/*
    The unrestricted information matrix is for a vector 
    autoregression model is

     T*[Inverse(I(n) x (A'))*(M(n,n) + I(n*n))*Inverse(I(n) x A)]

    where M(n,n) is the commutation matrix and I(k) is the k x k
    identitity matrix.  The notation B x C denotes the kronecker
    product.  A is an n x n invertiable matrix.  Returns the 
    information matrix divided by T.
*
TMatrix InformationMatrix(TMatrix A)
{
 int i;
 TMatrix X1, X2, X3;
 X1=KroneckerProduct((TMatrix)NULL,X2=IdentityMatrix((TMatrix)NULL,M_ROW(A)),X3=Inverse_LU((TMatrix)NULL,A)); 
 FreeMatrix(X2);
 FreeMatrix(X3);
 X2=CommutationMatrix((TMatrix)NULL,M_ROW(A),M_COL(A));
 for (i=M_ROW(A)*M_COL(A)-1; i >=0; i--) X2[i][i]+=1.0;            
 X3=TransposeProductMM((TMatrix)NULL,X1,X2);                                                                     
 ProductMM(X2,X3,X1);
 FreeMatrix(X1);
 FreeMatrix(X3);
 return X2;
}

/*
   Data from a vector autoregression can be used to approximate
   the information matrix defined above.  The approximation is
   given by 

   T-1
   --- 
   \    
    |  Vec(Inverse(A') - D(k)'D(k)A)Vec(Inverse(A') - D(k)'D(k)A)'
   /
   ---
   k=0

   where D(k) is the kth row of D.  The routine returns the 
   approximation divided by T.  The matrix A is n x n and the 
   data matrix D is T x n.
*
TMatrix InformationMatrixData(TMatrix A, TMatrix D)
{
 TMatrix A_inverse_transpose, X, Y, Z;
 TVector v;
 int i, j, k;

 X=Inverse_LU((TMatrix)NULL,A);
 A_inverse_transpose=Transpose((TMatrix)NULL,X);
 Y=CreateMatrix(M_ROW(A),M_ROW(A));
 v=CreateVector(M_ROW(A)*M_ROW(A));
 Z=InitializeMatrix(CreateMatrix(V_DIM(v),V_DIM(v)),0.0);

 for (k=M_ROW(D)-1; k >= 0; k--)
  {
   for (i=M_ROW(A)-1; i >= 0; i--)
    for (j=M_COL(A)-1; j >= 0; j--)
     X[i][j]=D[k][i]*D[k][j];
   ProductMM(Y,X,A);
   SubtractMM(Y,A_inverse_transpose,Y);
   Vec(v,Y);
   for (i=V_DIM(v)-1; i >= 0; i--)
    for (j=V_DIM(v)-1; j >= 0; j--)
     Z[i][j]+=v[i]*v[j];
  }

 ProductMS(Z,Z,1.0/(double)M_ROW(D));
 FreeVector(v);
 FreeMatrix(Y);
 FreeMatrix(A_inverse_transpose);
 FreeMatrix(X);

 return Z;
}

/*
*
TWaldNormalization* CreateWaldNormalizationParameters(TMatrix A)
{
  TWaldNormalization *params;
  params=(TWaldNormalization*)malloc(sizeof(TWaldNormalization));
  if (!params)
    {
      MatrixError(SIZE_ERR);
      return (TWaldNormalization*)NULL;
     }
  params->A=EquateMatrix((TMatrix)NULL,A);
  params->F=(TMatrix)NULL;
  params->S=InformationMatrix(A);
  return params;
}

/*
   Normalize using the information matrix at the point A.

   A     - 3 x 3 matrix to be normalized.
   A_hat - 3 x 3 reference matrix.

   Chooses the sign of each column of A to minimize

          Vec(A - A_hat)' * I(A) * Vec(A - A_hat)

   where I(A) is the information matrix at A.
*
void NormalizeVariableWaldMethod(TMatrix A, TMatrix A_hat)
{
 int i, j, n=M_ROW(A), *r, *s, *t;
 PRECISION d1, d2;
 TMatrix X, S;
 TVector v; 
 r=(int*)malloc(n*sizeof(int));
 s=(int*)malloc(n*sizeof(int));
 X=CreateMatrix(n,n);
 v=CreateVector(n*n);
 if (!r || !s || !X || !v) 
  {
   printf("Out of Memory!\n");
   exit(0);
  }
 for (i=n-1; i >= 0; i--) r[i]=s[i]=0;
 SubtractMM(X,A,A_hat);
 for (j=0; j < n; j++)
  for (i=0; i < n; i++)
   v[j*n+i]=X[i][j];
 d1=InnerProduct(v,v,S=InformationMatrix(A));
 FreeMatrix(S);
 while (1)
  {
   for (i=n-1; i >= 0; i--)
    if (r[i] == 1)
      r[i]=0;
     else
      {
       r[i]=1;
       break;
      }
   if (i < 0) break;
   EquateMatrix(X,A);
   for (j=n-1; j >= 0; j--)
    if (r[j]) 
     for (i=n-1; i >= 0; i--) 
      X[i][j]=-X[i][j];
   SubtractMM(X,X,A_hat);
   for (j=0; j < n; j++)
    for (i=0; i < n; i++)
     v[j*n+i]=X[i][j];
   d2=InnerProduct(v,v,S=InformationMatrix(A));
   FreeMatrix(S);
   if (d2 < d1)
    {
     d1=d2;
     t=r;
     r=s;
     s=t;
    }
  }
 for (j=n-1; j >= 0; j--)
  if (s[j]) 
   for (i=n-1; i >= 0; i--) 
    A[i][j]=-A[i][j];
 free(s);
 free(r);
 FreeVector(v);
 FreeMatrix(X);
}

/*
   Normalize using the information matrix at the point A_hat.

   A     - n x n matrix to be normalized.
   A_hat - n x n reference matrix.
   S     - information matrix at A_hat.

   Chooses the sign of each column of A to minimize

          Vec(A - A_hat)' * S * Vec(A - A_hat)
*
void NormalizeWaldMethod(TMatrix A, TMatrix A_hat, TMatrix S)
{
 int i, j, m=M_ROW(A)*M_ROW(A), *r, *s;
 PRECISION d1, d2;
 TMatrix X;
 TVector v;
 
 r=(int*)malloc(M_ROW(A)*sizeof(int));
 s=(int*)malloc(M_ROW(A)*sizeof(int));
 X=CreateMatrix(M_ROW(A),M_ROW(A));
 v=CreateVector(m);
 if (!r || !s || !X || !v) 
  {
   printf("Out of Memory!\n");
   exit(0);
  }
 for (i=M_ROW(A)-1; i >= 0; i--) r[i]=s[i]=0;
 SubtractMM(X,A,A_hat);
 Vec(v,X);
 d1=InnerProduct(v,v,S);
 while (1)
  {
   for (i=M_ROW(A)-1; i >= 0; i--)
    if (r[i] == 1)
      r[i]=0;
     else
      {
       r[i]=1;
       break;
      }
   if (i < 0) break;
   for (j=M_ROW(A)-1; j >= 0; j--)
    if (r[j]) 
      for (i=M_ROW(A)-1; i >= 0; i--) X[i][j]=-A[i][j];
     else
      for (i=M_ROW(A)-1; i >= 0; i--) X[i][j]=A[i][j];
   SubtractMM(X,X,A_hat);
   Vec(v,X);
   d2=InnerProduct(v,v,S);
   if (d2 < d1)
    {
     d1=d2;
     memcpy(s,r,M_ROW(A)*sizeof(int));
    }
  }
 for (j=M_ROW(A)-1; j >= 0; j--)
  if (s[j]) 
   for (i=M_ROW(A)-1; i >= 0; i--) 
    A[i][j]=-A[i][j];
 free(s);
 free(r);
 FreeVector(v);
 FreeMatrix(X);
}


TWaldNormalization* CreateWaldNormalizationParameters_2ndDerivative(TMatrix A, TMatrix F, TMatrix Data, TMatrix Exg, int p)
{
  TWaldNormalization *params;
  params=(TWaldNormalization*)malloc(sizeof(TWaldNormalization));
  if (!params)
    {
      MatrixError(SIZE_ERR);
      return (TWaldNormalization*)NULL;
     }
  params->A=EquateMatrix((TMatrix)NULL,A);
  params->F=EquateMatrix((TMatrix)NULL,F);
  params->S=InformationMatrix_2ndDerivative(A,F,Data,Exg,p);
  return params;
}

TWaldNormalization* CreateWaldNormalizationParameters_1stDerivative(TMatrix A, TMatrix F, TMatrix Data, TMatrix Exg, int p)
{
  TWaldNormalization *params;
  params=(TWaldNormalization*)malloc(sizeof(TWaldNormalization));
  if (!params)
    {
      MatrixError(SIZE_ERR);
      return (TWaldNormalization*)NULL;
     }
  params->A=EquateMatrix((TMatrix)NULL,A);
  params->F=EquateMatrix((TMatrix)NULL,F);
  params->S=InformationMatrix_1stDerivative(A,F,Data,Exg,p);
  return params;
}

void Destruct_TWaldNormalization(TWaldNormalization *params)
{
  if (params)
    {
      FreeMatrix(params->A);
      FreeMatrix(params->F);
      FreeMatrix(params->S);
      free(params);
    }
}
/**********************/
